@@ -238,7 +238,7 @@ VOID MlmeAssocReqAction(
 		COPY_MAC_ADDR(pAd->MlmeAux.Bssid, ApAddr);
 
 		// allocate and send out AssocRsp frame
-		NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);	//Get an unused nonpaged memory
+		NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);	//Get an unused nonpaged memory
 		if (NStatus != NDIS_STATUS_SUCCESS)
 		{
 			DBGPRINT(RT_DEBUG_TRACE,"ASSOC - MlmeAssocReqAction() allocate memory failed \n");
@@ -474,7 +474,7 @@ VOID MlmeReassocReqAction(
 		RTMPCancelTimer(&pAd->MlmeAux.ReassocTimer);
 
 		// allocate and send out ReassocReq frame
-		NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);	//Get an unused nonpaged memory
+		NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);	//Get an unused nonpaged memory
 		if (NStatus != NDIS_STATUS_SUCCESS)
 		{
 			DBGPRINT(RT_DEBUG_TRACE,"ASSOC - MlmeReassocReqAction() allocate memory failed \n");
@@ -575,7 +575,7 @@ VOID MlmeDisassocReqAction(
     pDisassocReq = (PMLME_DISASSOC_REQ_STRUCT)(Elem->Msg);
 
     // allocate and send out DeassocReq frame
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
     {
         DBGPRINT(RT_DEBUG_TRACE, "ASSOC - MlmeDisassocReqAction() allocate memory failed\n");
@@ -674,16 +674,20 @@ VOID link_status_handler(
 	
 	const int assoc_size = sizeof(*ndis_assoc_info) + IW_CUSTOM_MAX;
 	assoc_info = kmalloc(assoc_size, MEM_ALLOC_FLAG);
-	if (!assoc_info) {
-              DBGPRINT(RT_DEBUG_TRACE, "couldn't allocate memory\n");
+
+	if (!assoc_info) 
+	{
+        DBGPRINT(RT_DEBUG_TRACE, "couldn't allocate memory\n");
 		return;
 	}
+
 	memset(assoc_info, 0, assoc_size);
 	ndis_assoc_info = (NDIS_802_11_ASSOCIATION_INFORMATION *)assoc_info;
        
 	res = miniport_query_info(pAd, OID_802_11_ASSOCIATION_INFORMATION,
 				  assoc_info, assoc_size);
-	if (res) {
+	if (res) 
+	{
 		DBGPRINT(RT_DEBUG_TRACE, "query assoc_info failed\n");
 		kfree(assoc_info);
 		return;
@@ -702,21 +706,24 @@ VOID link_status_handler(
 #endif
 
     wpa_assoc_info_req = kmalloc(IW_CUSTOM_MAX, MEM_ALLOC_FLAG);
-	if (!wpa_assoc_info_req) {
+
+	if (!wpa_assoc_info_req) 
+	{
 		DBGPRINT(RT_DEBUG_TRACE, "couldn't allocate memory\n");
-		kfree(wpa_assoc_info_req);
 		return;
 	}
 
     //send ReqIEs
     memset(wpa_assoc_info_req, 0, IW_CUSTOM_MAX);
 	p = wpa_assoc_info_req;
-	p += sprintf(p, "ASSOCINFO(ReqIEs=");
+	p += sprintf(p, "ASSOCINFO_ReqIEs=");
 	ies = ((char *)ndis_assoc_info) +
 		ndis_assoc_info->OffsetRequestIEs;
+		
 	for (i = 0; i < ndis_assoc_info->RequestIELength; i++)
-		p += sprintf(p, "%02x", ies[i]);
+		p += sprintf(p, "%c", ies[i]);
 
+    p += sprintf(p, "\0");
 	memset(&wrqu, 0, sizeof(wrqu));
     wrqu.data.length = p - wpa_assoc_info_req;
 	wrqu.data.flags = RT_REQIE_EVENT_FLAG;
@@ -724,9 +731,9 @@ VOID link_status_handler(
 	wireless_send_event(pAd->net_dev, IWEVCUSTOM, &wrqu, wpa_assoc_info_req);
 
     wpa_assoc_info_resp = kmalloc(IW_CUSTOM_MAX, MEM_ALLOC_FLAG);
-	if (!wpa_assoc_info_resp) {
+	if (!wpa_assoc_info_resp) 
+	{
 		DBGPRINT(RT_DEBUG_TRACE, "couldn't allocate memory\n");
-		kfree(wpa_assoc_info_resp);
 		return;
 	}
 
@@ -736,23 +743,26 @@ VOID link_status_handler(
 	p += sprintf(p, " RespIEs=");
 	ies = ((char *)ndis_assoc_info) +
 		ndis_assoc_info->OffsetResponseIEs;
+
 	for (i = 0; i < ndis_assoc_info->ResponseIELength; i++)
-		p += sprintf(p, "%02x", ies[i]);
-	p += sprintf(p, ")");
+		p += sprintf(p, "%c", ies[i]);
+
+	p += sprintf(p, "\0");
 
     memset(&wrqu, 0, sizeof(wrqu));
 	wrqu.data.length = p - wpa_assoc_info_resp;
 	wrqu.data.flags = RT_RESPIE_EVENT_FLAG;
 	wireless_send_event(pAd->net_dev, IWEVCUSTOM, &wrqu, wpa_assoc_info_resp);
 
-        memset(&wrqu, 0, sizeof(wrqu));
-        wrqu.data.flags = RT_ASSOCINFO_EVENT_FLAG;
-        wireless_send_event(pAd->net_dev, IWEVCUSTOM, &wrqu, NULL);
+    memset(&wrqu, 0, sizeof(wrqu));
+    wrqu.data.flags = RT_ASSOCINFO_EVENT_FLAG;
+    wireless_send_event(pAd->net_dev, IWEVCUSTOM, &wrqu, NULL);
 
 #if 0
 	/* we need 28 extra bytes for the format strings */
 	if ((ndis_assoc_info->RequestIELength +
-	     ndis_assoc_info->ResponseIELength + 28) > IW_CUSTOM_MAX) {
+	     ndis_assoc_info->ResponseIELength + 28) > IW_CUSTOM_MAX) 
+    {
 		//WARNING("information element is too long! (%u,%u),"
 		//	"association information dropped",
 		//	ndis_assoc_info->RequestIELength,
@@ -764,7 +774,8 @@ VOID link_status_handler(
 	}
 
 	wpa_assoc_info = kmalloc(IW_CUSTOM_MAX, MEM_ALLOC_FLAG);
-	if (!wpa_assoc_info) {
+	if (!wpa_assoc_info) 
+	{
 		DBGPRINT(RT_DEBUG_TRACE, "couldn't allocate memory\n");
 		kfree(assoc_info);
 		return;
@@ -800,6 +811,7 @@ VOID link_status_handler(
 }
 #endif
 
+
 /*
     ==========================================================================
     Description:
@@ -1177,7 +1189,7 @@ VOID Cls3errAction(
     USHORT              NStatus;
     
     // allocate memory
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
         return;
     
