@@ -132,7 +132,7 @@ VOID MlmeAuthReqAction(
         Status = MLME_SUCCESS;
 
         // allocate and send out AuthReq frame
-        NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+        NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
         if (NStatus != NDIS_STATUS_SUCCESS)
         {
             DBGPRINT(RT_DEBUG_TRACE, "AUTH - MlmeAuthReqAction() allocate memory failed\n");
@@ -207,7 +207,7 @@ VOID PeerAuthRspAtSeq2Action(
                     Seq++;
                     RemoteStatus = MLME_SUCCESS;
                     // allocate and send out AuthRsp frame
-                    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+                    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
                     if (NStatus != NDIS_STATUS_SUCCESS)
                     {
                         DBGPRINT(RT_DEBUG_TRACE, "AUTH - PeerAuthRspAtSeq2Action() allocate memory fail\n");
@@ -323,7 +323,7 @@ VOID MlmeDeauthReqAction(
     pInfo = (MLME_DEAUTH_REQ_STRUCT *)Elem->Msg;
 
     // allocate and send out DeauthReq frame
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
     {
         DBGPRINT(RT_DEBUG_TRACE, "AUTH - MlmeDeauthReqAction() allocate memory fail\n");
@@ -404,7 +404,7 @@ VOID Cls2errAction(
     USHORT          NStatus;
     
     // allocate memory
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
         return;
 
