@@ -345,18 +345,19 @@ VOID CntlOidSsidProc(
 			// case 1.3. already connected to the SSID with highest RSSI.
 			DBGPRINT(RT_DEBUG_TRACE, "CNTL - already with this BSSID. ignore this SET_SSID request\n");
 
-
-			// We only check if same to the BSSID with highest RSSI.
-			// If roaming of same SSID required, we still do the reconnection.
-			// same BSSID, go back to idle state directly
-			if (pAd->MlmeAux.CurrReqIsFromNdis)
-			{
-			}
 			pAd->Mlme.CntlMachine.CurrState = CNTL_IDLE;
 		} 
 	} 
 	else if (INFRA_ON(pAd)) 
 	{
+		if (!SSID_EQUAL(pAd->PortCfg.Ssid, pAd->PortCfg.SsidLen, pAd->MlmeAux.Ssid, pAd->MlmeAux.SsidLen))
+		{
+			//
+			// Different SSID means not Roaming case, so we let LinkDown() to Indicate a disconnect event.
+			// 
+			pAd->MlmeAux.CurrReqIsFromNdis = TRUE;
+		}
+
 		// case 2. active INFRA association existent
 		//	  roaming is done within miniport driver, nothing to do with configuration
 		//	  utility. so upon a new SET(OID_802_11_SSID) is received, we just 
@@ -980,7 +981,11 @@ VOID LinkUp(
 		{	
 			// Remove all WPA keys
 			RTMPWPARemoveAllKeys(pAd);		
+			pAd->PortCfg.PortSecured = WPA_802_1X_PORT_NOT_SECURED;
 		}
+		// store Bssid
+		COPY_MAC_ADDR(pAd->PortCfg.LastBssid, pAd->PortCfg.Bssid); 
+
 		
 		// NOTE:
 		// the decision of using "short slot time" or not may change dynamically due to
@@ -1074,7 +1079,7 @@ VOID LinkDown(
 #ifdef	SINGLE_ADHOC_LINKUP
 		OPSTATUS_CLEAR_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED);
 		// clean up previous SCAN result, add current BSS back to table if any
-		BssTableDeleteEntry(&pAd->PortCfg.ScanTab, &(pAd->PortCfg.Bssid), pAd->PortCfg.Channel);
+		BssTableDeleteEntry(&pAd->ScanTab, &(pAd->PortCfg.Bssid), pAd->PortCfg.Channel);
 #else
 		if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RADIO_OFF))
 		{
@@ -1100,10 +1105,17 @@ VOID LinkDown(
 		{
 			// Set linkdown timer
 			RTMPSetTimer(pAd, &pAd->Mlme.LinkDownTimer, 10000);
+					
+			//
+			// If disassociation request is from NDIS, then we don't need to delete BSSID frmo entry.
+			// Otherwise lost beacon or receive De-Authentication from AP, 
+			// then we should delete BSSID from BssTable.
+			// If we don't delete from entry, roaming will fail.
+			//
+			BssTableDeleteEntry(&pAd->ScanTab, pAd->PortCfg.Bssid, pAd->PortCfg.Channel);
+
 		}
 				
-		BssTableDeleteEntry(&pAd->ScanTab, pAd->PortCfg.Bssid, pAd->PortCfg.Channel);
-
 		// restore back to - 
 		//		1. long slot (20 us) or short slot (9 us) time
 		//		2. turn on/off RTS/CTS and/or CTS-to-self protection
