@@ -97,7 +97,7 @@
 #define SCAN_PASSIVE                     18
 #define SCAN_ACTIVE                      19
 #define FAST_SCAN_ACTIVE                 24		// scan with probe request, and wait beacon and probe response
-#define MAX_LEN_OF_MLME_QUEUE            20
+#define MAX_LEN_OF_MLME_QUEUE            40
 
 
 // bit definition of the 2-byte pBEACON->Capability field
@@ -224,7 +224,7 @@ typedef struct PACKED {
     USHORT      CfpDurRemaining;
 } CF_PARM, *PCF_PARM;
 
-typedef	struct	_CIPHER_SUITE	{
+typedef	struct	PACKED _CIPHER_SUITE	{
 	NDIS_802_11_ENCRYPTION_STATUS	PairCipher;		// Unicast cipher 1, this one has more secured cipher suite
 	NDIS_802_11_ENCRYPTION_STATUS	PairCipherAux;	// Unicast cipher 2 if AP announce two unicast cipher suite
 	NDIS_802_11_ENCRYPTION_STATUS	GroupCipher;	// Group cipher
@@ -301,7 +301,8 @@ typedef struct {
 	NDIS_802_11_FIXED_IEs	FixIEs;
 	NDIS_802_11_AUTHENTICATION_MODE	AuthModeAux;	// Addition mode for WPA2 / WPA capable AP
 	NDIS_802_11_AUTHENTICATION_MODE	AuthMode;	
-	UCHAR                           AuthBitMode;
+	UCHAR                           AuthBitMode;        // authTypeFlags for WPA, a bitwise OR
+	UCHAR                           PairCipherBitMode;  // encTypeFlags for WPA, a bitwise OR
 	NDIS_802_11_WEP_STATUS	WepStatus;				// Unicast Encryption Algorithm extract from VAR_IE
 	UCHAR					VarIELen;				// Length of next VIE include EID & Length
 	UCHAR					VarIEs[MAX_VIE_LEN];
