@@ -54,6 +54,9 @@
 #ifdef DBG
 #define RTPRIV_IOCTL_BBP                            (SIOCIWFIRSTPRIV + 0x03)
 #define RTPRIV_IOCTL_MAC                            (SIOCIWFIRSTPRIV + 0x05)
+#ifdef RALINK_ATE
+#define RTPRIV_IOCTL_E2P                            (SIOCIWFIRSTPRIV + 0x11)
+#endif
 #endif
 #define RTPRIV_IOCTL_STATISTICS                     (SIOCIWFIRSTPRIV + 0x09)
 #define RTPRIV_IOCTL_GSITESURVEY					(SIOCIWFIRSTPRIV + 0x0D)
@@ -133,8 +136,6 @@
 #define OID_802_11_PMKID                            0x0620
 #define RT_OID_WPA_SUPPLICANT_SUPPORT               0x0621
 #define RT_OID_WE_VERSION_COMPILED                  0x0622
-#define OID_SET_WSC_IE_PROBE_REQ                    0x0624
-#define OID_802_11_RCV_BEACON                       0x0625
 //#endif
 
 #define OID_802_11_ENCRYPTION_STATUS                OID_802_11_WEP_STATUS
@@ -404,7 +405,7 @@ typedef  ULONG  NDIS_802_11_KEY_INDEX;
 typedef ULONGLONG   NDIS_802_11_KEY_RSC;
 
 // Key mapping keys require a BSSID
-typedef struct _NDIS_802_11_KEY
+typedef struct PACKED _NDIS_802_11_KEY
 {
     ULONG           Length;             // Length of this structure
     ULONG           KeyIndex;           
@@ -490,6 +491,9 @@ typedef enum _NDIS_802_11_AUTHENTICATION_MODE
 #define WPA1PSKAKMBIT   4
 #define WPA2PSKAKMBIT   8
 
+#define TKIPBIT         0x01
+#define CCMPBIT         0x02
+
 typedef UCHAR NDIS_802_11_RATES[NDIS_802_11_LENGTH_RATES];        // Set of 8 data rates
 typedef UCHAR NDIS_802_11_RATES_EX[NDIS_802_11_LENGTH_RATES_EX];  // Set of 16 data rates
 
@@ -705,7 +709,7 @@ typedef struct _RT_802_11_STA_CONFIG {
     ULONG   SystemErrorBitmap;  // ignore upon SET, return system error upon QUERY
 } RT_802_11_STA_CONFIG, *PRT_802_11_STA_CONFIG;
 
-typedef struct _RT_VERSION_INFO{
+typedef struct PACKED _RT_VERSION_INFO{
     UCHAR       DriverVersionW;
     UCHAR       DriverVersionX;
     UCHAR       DriverVersionY;
