@@ -23,7 +23,7 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
  *                                                                       * 
  ************************************************************************
- 
+
 	Module Name:
 	rtmp_def.h
 
@@ -66,9 +66,9 @@
 //
 // Entry number for each DMA descriptor ring
 //
-#define TX_RING_SIZE			32
+#define TX_RING_SIZE			64
 #define PRIO_RING_SIZE			16
-#define RX_RING_SIZE			32
+#define RX_RING_SIZE			64
 #define	BEACON_RING_SIZE		2
 #define MGMT_RING_SIZE			PRIO_RING_SIZE
 #define PRIO_BUFFER_SIZE		1024	// 2048
@@ -87,8 +87,8 @@
 #define	MAX_CLIENT				2
 
 #define LOCAL_TXBUF_SIZE		2048
-#define TXD_SIZE				64
-#define RXD_SIZE				64
+#define TXD_SIZE				24
+#define RXD_SIZE				24
 #define TX_DMA_1ST_BUFFER_SIZE	64	  // only the 1st physical buffer is pre-allocated
 #define MGMT_DMA_BUFFER_SIZE	2048
 #define RX_DMA_BUFFER_SIZE      4096
@@ -125,7 +125,7 @@
 #define fRTMP_ADAPTER_RESET_PIPE_IN_PROGRESS 0x00200000
 #define fRTMP_ADAPTER_SCAN_CHANNEL_IN_PROGRESS 0x04000000
 #define	fRTMP_ADAPTER_RADIO_MEASUREMENT		0x08000000
-
+#define fRTMP_ADAPTER_REMOVE_IN_PROGRESS	0x10000000
 #define fRTMP_ADAPTER_MEDIA_STATE_CHANGE	0x20000000
 
 
@@ -177,6 +177,9 @@
 #define	fRTUSB_BULK_OUT_DATA_FRAG_2				0x00000080
 #define	fRTUSB_BULK_OUT_DATA_FRAG_3				0x00000100
 #define	fRTUSB_BULK_OUT_DATA_FRAG_4				0x00000200
+#ifdef RALINK_ATE
+#define	fRTUSB_BULK_OUT_DATA_ATE				0x00100000
+#endif
 
 //
 //	AP's client table operation status flags
@@ -655,6 +658,7 @@
 #define RFIC_2528				2  //B/G
 #define RFIC_5225				3  //A/B/G
 #define RFIC_2527				4  //B/G
+#define RFIC_3020               5  //B/G
 
 // LED Status.
 #define LED_LINK_DOWN               0
@@ -690,7 +694,8 @@
 
 // 802.1X controlled port definition
 #define	WPA_802_1X_PORT_SECURED			1
-#define	WPA_802_1X_PORT_NOT_SECURED		2
+#define WPA_802_1X_PORT_PASS_4_WAY  	2 
+#define	WPA_802_1X_PORT_NOT_SECURED		3
 
 #define	PAIRWISE_KEY			1
 #define	GROUP_KEY				2
@@ -805,6 +810,14 @@
 
 typedef UCHAR ADDRESS[MAC_ADDRESS_LENGTH];
 
+#ifdef RALINK_ATE
+#define	ATE_STASTOP                 0   // Stop STA
+#define	ATE_STASTART                1   // Start STA
+#define	ATE_TXCONT                  2   // Continuous Transmit
+#define	ATE_TXCARR                  3   // Transmit Carrier
+#define	ATE_TXFRAME                 4   // Transmit Frames
+#define	ATE_RXFRAME                 5   // Receive Frames
+#endif	// RALINK_ATE
 
 //-------------------
 // VID/PID
@@ -815,6 +828,7 @@ typedef UCHAR ADDRESS[MAC_ADDRESS_LENGTH];
  {USB_DEVICE(0x148f,0x2671)}, /* Ralink */      \
  {USB_DEVICE(0x18e8,0x6196)}, /* Qcom */        \
  {USB_DEVICE(0x18e8,0x6229)}, /* Qcom */        \
+ {USB_DEVICE(0x18e8,0x6238)}, /* Qcom */        \
  {USB_DEVICE(0x1044,0x8008)}, /* Gigabyte */    \
  {USB_DEVICE(0x14b2,0x3c22)}, /* Conceptronic */    \
  {USB_DEVICE(0x0db0,0x6877)}, /* MSI */         \
@@ -828,6 +842,18 @@ typedef UCHAR ADDRESS[MAC_ADDRESS_LENGTH];
  {USB_DEVICE(0x1631,0xc019)}, /* Billionton */  \
  {USB_DEVICE(0x1044,0x800a)}, /* Gigabyte */	\
  {USB_DEVICE(0x1690,0x0722)}, /* Askey */		\
+ {USB_DEVICE(0x0df6,0x9712)}, /* Sitecom WL-113 v1 002 */	\
+ {USB_DEVICE(0x0df6,0x90ac)}, /* Sitecom */  \
+ {USB_DEVICE(0x148f,0x9021)}, /* Amigo */	\
+ {USB_DEVICE(0x0eb0,0x9021)}, /* Amigo */	\
+ {USB_DEVICE(0x0411,0x00D8)}, /* BUFFALO WLI-U2-SG54HP */	\
+ {USB_DEVICE(0x0586,0x3415)}, /* ZyXEL */	\
+ {USB_DEVICE(0x0b05,0x1723)}, /* ASUS */  \
+ {USB_DEVICE(0x0b05,0x1724)}, /* ASUS */	\
+ {USB_DEVICE(0x050d,0x705a)}, /* Belkin */  \
+ {USB_DEVICE(0x07d1,0x3c03)}, /* D-Link */  \
+ {USB_DEVICE(0x13b1,0x0023)}, /* Linksys */  \
+ {USB_DEVICE(0x13b1,0x0020)}, /* Linksys WUS54GC */ \
  {USB_DEVICE(0,0)}} /* end marker */
 
 
