@@ -64,8 +64,8 @@ VOID RTUSBBulkOutDataPacketComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 	PRTMP_ADAPTER	pAd;
 	NTSTATUS		status;
 	UCHAR			BulkOutPipeId;
-	ULONG			IrqFlags;
-
+	unsigned long	IrqFlags;
+	
 	DBGPRINT_RAW(RT_DEBUG_INFO, "--->RTUSBBulkOutDataPacketComplete\n");
 	pTxContext= (PTX_CONTEXT)pUrb->context;
 	pAd = pTxContext->pAd;
@@ -170,7 +170,7 @@ VOID RTUSBBulkOutNullFrameComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 	PRTMP_ADAPTER	pAd;
 	PTX_CONTEXT		pNullContext;
 	NTSTATUS		status;
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	pNullContext= (PTX_CONTEXT)pUrb->context;
 	pAd = pNullContext->pAd;
@@ -221,7 +221,7 @@ VOID RTUSBBulkOutRTSFrameComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 	PRTMP_ADAPTER	pAd;
 	PTX_CONTEXT		pRTSContext;
 	NTSTATUS		status;
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	pRTSContext= (PTX_CONTEXT)pUrb->context;
 	pAd = pRTSContext->pAd;
@@ -273,7 +273,7 @@ VOID RTUSBBulkOutMLMEPacketComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 	PTX_CONTEXT			pMLMEContext;
 	PRTMP_ADAPTER		pAd;
 	NTSTATUS			status;
-	ULONG				IrqFlags;
+	unsigned long	IrqFlags;
 	
 	pMLMEContext= (PTX_CONTEXT)pUrb->context;
 	pAd = pMLMEContext->pAd;
@@ -298,6 +298,7 @@ VOID RTUSBBulkOutMLMEPacketComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 	// Reset MLME context flags
 	pMLMEContext->IRPPending	= FALSE;
 	pMLMEContext->InUse 		= FALSE;
+	pMLMEContext->bWaitingBulkOut =FALSE;
 	
 	if (status == USB_ST_NOERROR)
 	{
@@ -318,6 +319,10 @@ VOID RTUSBBulkOutMLMEPacketComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 		}
 	}
 #endif
+	pMLMEContext = &pAd->MLMEContext[pAd->PrioRingFirstIndex];
+
+	if ( (pAd->PrioRingTxCnt >= 1) && (pMLMEContext->bWaitingBulkOut == TRUE))
+		RTUSB_SET_BULK_FLAG(pAd, fRTUSB_BULK_OUT_MLME);
 
 	NdisAcquireSpinLock(&pAd->BulkOutLock[0], IrqFlags);
 	pAd->BulkOutPending[0] = FALSE;
@@ -337,8 +342,8 @@ VOID RTUSBBulkOutPsPollComplete(purbb_t pUrb,struct pt_regs *pt_regs)
 	PRTMP_ADAPTER	pAd;
 	PTX_CONTEXT		pPsPollContext;
 	NTSTATUS		status;
-	ULONG			IrqFlags;
-
+	unsigned long	IrqFlags;
+	
 	pPsPollContext= (PTX_CONTEXT)pUrb->context;
 	pAd = pPsPollContext->pAd;
 
@@ -452,6 +457,7 @@ VOID RTUSBBulkRxComplete(purbb_t pUrb, struct pt_regs *pt_regs)
 		
 		case -ECONNRESET:		// async unlink
 		case -ESHUTDOWN:		// hardware gone = -108
+		case -EPROTO:			// protocol error
 			pUrb = NULL;
 			DBGPRINT(RT_DEBUG_ERROR,"==> RTUSBBulkRxComplete Error code = %d\n", status);
 			//tasklet_schedule(&pAd->rx_bh);
@@ -553,7 +559,7 @@ VOID	RTUSBBulkOutDataPacket(
 	PTX_CONTEXT	pTxContext;
 	PURB		pUrb;
 	int 		ret = 0;
-	ULONG		IrqFlags;
+	unsigned long	IrqFlags;
 	
 	NdisAcquireSpinLock(&pAd->BulkOutLock[BulkOutPipeId], IrqFlags);
 	if (pAd->BulkOutPending[BulkOutPipeId] == TRUE)
@@ -663,7 +669,7 @@ VOID	RTUSBBulkOutNullFrame(
 	PTX_CONTEXT	pNullContext = &(pAd->NullContext);
 	PURB		pUrb;
 	int 		ret = 0;
-	ULONG		IrqFlags;
+	unsigned long	IrqFlags;
 	
 	NdisAcquireSpinLock(&pAd->BulkOutLock[0], IrqFlags);
 	if (pAd->BulkOutPending[0] == TRUE)
@@ -717,7 +723,7 @@ VOID	RTUSBBulkOutRTSFrame(
 	PTX_CONTEXT	pRTSContext = &(pAd->RTSContext);
 	PURB		pUrb;
 	int 		ret = 0;
-	ULONG		IrqFlags;
+	unsigned long	IrqFlags;
 	UCHAR		PipeID=0;
 	
 	if (RTUSB_TEST_BULK_FLAG(pAd, fRTUSB_BULK_OUT_DATA_NORMAL_4))
@@ -782,7 +788,7 @@ VOID	RTUSBBulkOutMLMEPacket(
 	PTX_CONTEXT		pMLMEContext;
 	PURB			pUrb;
 	int 			ret = 0;
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	pMLMEContext = &pAd->MLMEContext[Index];
 	
@@ -843,7 +849,7 @@ VOID	RTUSBBulkOutPsPoll(
 	PTX_CONTEXT		pPsPollContext = &(pAd->PsPollContext);
 	PURB			pUrb;
 	int 			ret = 0;
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	NdisAcquireSpinLock(&pAd->BulkOutLock[0], IrqFlags);
 	if (pAd->BulkOutPending[0] == TRUE)
@@ -932,6 +938,25 @@ VOID	RTUSBBulkReceive(
 	return;
 }
 
+VOID RTUSBBulkRxHandle(
+	IN unsigned long data)
+{
+	purbb_t 		pUrb = (purbb_t)data;
+	PRTMP_ADAPTER	pAd;
+	PRX_CONTEXT 	pRxContext;
+
+
+	pRxContext = (PRX_CONTEXT)pUrb->context;
+	pAd = pRxContext->pAd;
+	
+	/* device had been closed */
+	if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_REMOVE_IN_PROGRESS)) 
+		return;
+
+	RTUSBRxPacket(data);
+	return;
+}
+
 /*
 	========================================================================
 	
@@ -956,6 +981,9 @@ VOID	RTUSBKickBulkOut(
 		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS)) &&
 		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_IN_PROGRESS)) &&
 		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_PIPE_IN_PROGRESS)) &&
+#ifdef RALINK_ATE			
+		!(pAd->ate.Mode != ATE_STASTART) &&
+#endif					
 		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RADIO_OFF)))
 	{
 
@@ -1060,6 +1088,22 @@ VOID	RTUSBKickBulkOut(
 		}
 	}
 	
+#ifdef RALINK_ATE		//2006/06/11	
+	//If the mode is in ATE mode.
+	else if((pAd->ate.Mode != ATE_STASTART) &&
+		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_NIC_NOT_EXIST)) &&
+		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS)) &&
+		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_IN_PROGRESS)) &&
+		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_PIPE_IN_PROGRESS)) &&						
+		!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RADIO_OFF)))
+	{
+		if (RTUSB_TEST_BULK_FLAG(pAd, fRTUSB_BULK_OUT_DATA_ATE))
+		{
+			ATE_RTUSBBulkOutDataPacket(pAd);
+		}
+	}	
+#endif	
+	
 	DBGPRINT_RAW(RT_DEBUG_INFO, "<---RTUSBKickBulkOut\n");
 }
 
@@ -1081,7 +1125,7 @@ VOID	RTUSBCleanUpDataBulkOutQueue(
 {
 	UCHAR			Idx;
 	PTX_CONTEXT 	pTxContext;			
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	DBGPRINT(RT_DEBUG_TRACE, "--->CleanUpDataBulkOutQueue\n");
 
@@ -1119,7 +1163,7 @@ VOID	RTUSBCleanUpDataBulkOutQueue(
 VOID	RTUSBCleanUpMLMEBulkOutQueue(
 	IN	PRTMP_ADAPTER	pAd)
 {
-	ULONG			IrqFlags;
+	unsigned long	IrqFlags;
 	
 	DBGPRINT(RT_DEBUG_TRACE, "--->CleanUpMLMEBulkOutQueue\n");
 
@@ -1196,8 +1240,8 @@ VOID	RTUSBCancelPendingBulkOutIRP(
 	PTX_CONTEXT		pPsPollContext;
 	PTX_CONTEXT		pRTSContext;
 	UINT			i, Idx;
-	ULONG			IrqFlags;
-
+	unsigned long	IrqFlags;
+	
 	for (Idx = 0; Idx < 4; Idx++)
 	{
 		for (i = 0; i < TX_RING_SIZE; i++)
@@ -1298,3 +1342,125 @@ VOID	RTUSBCancelPendingIRPs(
 	RTUSBCancelPendingBulkOutIRP(pAd);
 }
 
+#ifdef RALINK_ATE
+VOID ATE_RTUSBBulkOutDataPacketComplete(purbb_t pUrb, struct pt_regs *pt_regs)
+{
+	PRTMP_ADAPTER	pAd;
+	PTX_CONTEXT		pNullContext;
+	NTSTATUS		status;
+	unsigned long	IrqFlags;
+	ULONG			OldValue;
+	
+	pNullContext= (PTX_CONTEXT)pUrb->context;
+	pAd = pNullContext->pAd;
+
+	DBGPRINT_RAW(RT_DEBUG_INFO, "--->ATE_RTUSBBulkOutDataPacketComplete\n");
+
+	// Reset Null frame context flags
+	pNullContext->IRPPending = FALSE;
+	pNullContext->InUse = FALSE;
+	
+	status = pUrb->status;
+	
+	if (status == USB_ST_NOERROR)
+	{
+		// Don't worry about the queue is empty or not, this function will check itself
+		RTMPDeQueuePacket(pAd, 0);
+	}
+#if 1	// STATUS_OTHER
+	else
+	{
+		if ((!RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_RESET_IN_PROGRESS)) &&
+			(!RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS)) &&
+			(!RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_NIC_NOT_EXIST)) &&
+			(!RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_BULKOUT_RESET)))
+		{
+			DBGPRINT_RAW(RT_DEBUG_ERROR, "Bulk Out Null Frame Failed\n");
+			RTMP_SET_FLAG(pAd, fRTMP_ADAPTER_BULKOUT_RESET);
+			RTUSBEnqueueInternalCmd(pAd, RT_OID_USB_RESET_BULK_OUT);
+		}
+	}
+#endif	
+
+	if (atomic_read(&pAd->BulkOutRemained) > 0)
+	{			
+		atomic_dec(&pAd->BulkOutRemained);
+		DBGPRINT(RT_DEBUG_INFO, "Bulk Out Remained = %d\n", atomic_read(&pAd->BulkOutRemained));
+	}
+	
+	// 1st - Transmit Success
+	OldValue = pAd->WlanCounters.TransmittedFragmentCount.vv.LowPart;
+	pAd->WlanCounters.TransmittedFragmentCount.vv.LowPart++;
+	if (pAd->WlanCounters.TransmittedFragmentCount.vv.LowPart < OldValue)
+	{
+		pAd->WlanCounters.TransmittedFragmentCount.vv.HighPart++;
+	}
+	
+	if(((pAd->ContinBulkOut == TRUE ) ||(atomic_read(&pAd->BulkOutRemained) > 0)) && (pAd->ate.Mode != ATE_STASTART))
+	{	
+	
+		DBGPRINT(RT_DEBUG_INFO, "ContinBulkOut = TRUE \n");
+		RTUSB_SET_BULK_FLAG(pAd, fRTUSB_BULK_OUT_DATA_ATE);
+	}	
+	else
+	{
+		RTUSB_CLEAR_BULK_FLAG(pAd, fRTUSB_BULK_OUT_DATA_ATE);
+	}
+
+	NdisAcquireSpinLock(&pAd->BulkOutLock[0], IrqFlags);
+	pAd->BulkOutPending[0] = FALSE;
+	NdisReleaseSpinLock(&pAd->BulkOutLock[0], IrqFlags);	
+	// Always call Bulk routine, even reset bulk.
+	// The protectioon of rest bulk should be in BulkOut routine
+	RTUSBKickBulkOut(pAd);
+
+	DBGPRINT_RAW(RT_DEBUG_INFO, "<---ATE_RTUSBBulkOutDataPacketComplete\n");
+
+}
+
+
+VOID ATE_RTUSBBulkOutDataPacket(	
+	IN	PRTMP_ADAPTER	pAd)
+{
+	PTX_CONTEXT		pNullContext = &(pAd->NullContext);
+	PURB			pUrb;
+	int 			ret = 0;
+	unsigned long	IrqFlags;
+
+
+	ATE_RTMPSendNullFrame(pAd);
+	
+	DBGPRINT_RAW(RT_DEBUG_INFO, "--->ATE_RTUSBBulkOutDataPacket \n");	
+	NdisAcquireSpinLock(&pAd->BulkOutLock[0], IrqFlags);
+	if (pAd->BulkOutPending[0] == TRUE)
+	{
+		DBGPRINT_RAW(RT_DEBUG_INFO, "--->ATE_RTUSBBulkOutDataPacket BulkOutPending \n");
+		NdisReleaseSpinLock(&pAd->BulkOutLock[0], IrqFlags);
+		return;
+	}
+	pAd->BulkOutPending[0] = TRUE;
+	NdisReleaseSpinLock(&pAd->BulkOutLock[0], IrqFlags);
+
+	// Increase Total transmit byte counter
+	pAd->RalinkCounters.TransmittedByteCount +=  pNullContext->BulkOutSize;
+
+	DBGPRINT_RAW(RT_DEBUG_INFO, "--->ATE_RTUSBBulkOutDataPacket \n");
+
+	// Clear Null frame bulk flag
+	RTUSB_CLEAR_BULK_FLAG(pAd, fRTUSB_BULK_OUT_DATA_ATE);	
+
+	// Init Tx context descriptor
+	RTUSBInitTxDesc(pAd, pNullContext, 0, ATE_RTUSBBulkOutDataPacketComplete);
+	pNullContext->IRPPending = TRUE;
+
+	pUrb = pNullContext->pUrb;
+	if((ret = rtusb_submit_urb(pUrb))!=0)
+	{
+		DBGPRINT(RT_DEBUG_ERROR,"Submit Tx URB failed %d\n", ret);
+		return;
+	}	
+	
+	DBGPRINT_RAW(RT_DEBUG_INFO, "<---ATE_RTUSBBulkOutDataPacket \n");
+	return;
+}
+#endif
