@@ -43,7 +43,6 @@ UCHAR	RSN_OUI[] = {0x00, 0x0f, 0xac};
 UCHAR   WME_INFO_ELEM[]  = {0x00, 0x50, 0xf2, 0x02, 0x00, 0x01};
 UCHAR   WME_PARM_ELEM[] = {0x00, 0x50, 0xf2, 0x02, 0x01, 0x01};
 UCHAR   RALINK_OUI[]  = {0x00, 0x0c, 0x43};
-UCHAR   WSC_OUI[] = {0x00, 0x50, 0xf2, 0x04};
 
 /* 
     ==========================================================================
@@ -770,29 +769,9 @@ BOOLEAN PeerBeaconAndProbeRspSanity(
 					pEdcaParm->Cwmax[QID_AC_VO] = CW_MAX_IN_BITS-1;
 					pEdcaParm->Txop[QID_AC_VO]	= 48;	// AC_VO: 48*32us ~= 1.5ms
 				}
-#if WPA_SUPPLICANT_SUPPORT
-				else if (NdisEqualMemory(pEid->Octet, WSC_OUI, 4) && (SubType == SUBTYPE_BEACON) && (pAd->PortCfg.Send_Beacon == TRUE))
-				{
-					union iwreq_data wrqu;
-					char buf[560] = {0};
-                                    
-					pAd->PortCfg.WscIEBeacon.ValueLen = pEid->Len - 4;
-					NdisMoveMemory(pAd->PortCfg.WscIEBeacon.Value, &pEid->Octet[4], pEid->Len - 4);
-                                    
-                
-					//Send WSC beacon to wpa_supplicant
-					memset(&wrqu, 0, sizeof(wrqu));
-					sprintf(buf, "WSCBEACON=<%d>", pAd->PortCfg.WscIEBeacon.ValueLen);
-					wrqu.data.length = strlen(buf);
-					NdisMoveMemory(&buf[wrqu.data.length], pAd->PortCfg.WscIEBeacon.Value, pAd->PortCfg.WscIEBeacon.ValueLen);
-					wrqu.data.length += pAd->PortCfg.WscIEBeacon.ValueLen;
-					wireless_send_event(pAd->net_dev, IWEVCUSTOM, &wrqu, buf);
-					//COPY_MAC_ADDR(pAd->PortCfg.MBSSID[pAd->IoctlIF].WscIEProbRspAddr2, pAddr2);
-					//return FALSE;
-				}
-#endif
 				else	
 				{
+#if 0   // Let pVIE dedicated to wpa1 and wpa2 ie				
 					// Gemtek ask us to pass other vendor's IE for their applications
 					Ptr = (PUCHAR) pVIE;
 					
@@ -806,6 +785,7 @@ BOOLEAN PeerBeaconAndProbeRspSanity(
 
 					NdisMoveMemory(Ptr + *LengthVIE, &pEid->Eid, pEid->Len + 2);
 					*LengthVIE += (pEid->Len + 2);
+#endif					
 				}
 			
 				DBGPRINT(RT_DEBUG_INFO, "PeerBeaconAndProbeRspSanity - Receive IE_WPA\n");
@@ -1173,8 +1153,8 @@ NDIS_STATUS	RTMPWPAWepKeySanity(
 	NDIS_STATUS			Status = NDIS_STATUS_SUCCESS;
 	BOOLEAN 			bTxKey; 		// Set the key as transmit key
 	BOOLEAN 			bPairwise;		// Indicate the key is pairwise key
-	UCHAR			CipherAlg;
-	UINT			i;
+//	UCHAR			CipherAlg;
+//	UINT			i;
 
 	pKey = (PNDIS_802_11_KEY) pBuf;
 	KeyIdx = pKey->KeyIndex & 0x0fffffff;
@@ -1226,9 +1206,6 @@ NDIS_STATUS	RTMPWPAWepKeySanity(
     //always use BSS0=0
 //	AsicAddSharedKeyEntry(pAd, 0, (UCHAR)KeyIdx, CipherAlg, pAd->SharedKey[KeyIdx].Key, NULL, NULL);
 
-	
-     	pAd->PortCfg.PortSecured = WPA_802_1X_PORT_SECURED;	//For Test
-
 	return (Status);
 }
 
