@@ -151,6 +151,12 @@ VOID BeaconTimeout(
     RTMP_ADAPTER *pAd = (RTMP_ADAPTER *)data;
 
     DBGPRINT(RT_DEBUG_TRACE,"SYNC - BeaconTimeout\n");
+
+	// Do nothing if the driver is starting halt state.
+	// This might happen when timer already been fired before cancel timer with mlmehalt
+	if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS))
+		return;
+    
     MlmeEnqueue(pAd, SYNC_STATE_MACHINE, MT2_BEACON_TIMEOUT, 0, NULL);
     RTUSBMlmeUp(pAd);
 }
@@ -167,6 +173,12 @@ VOID ScanTimeout(
     RTMP_ADAPTER *pAd = (RTMP_ADAPTER *)data;
 
     DBGPRINT(RT_DEBUG_INFO,"SYNC - Scan Timeout \n");
+
+  	// Do nothing if the driver is starting halt state.
+	// This might happen when timer already been fired before cancel timer with mlmehalt
+	if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS))
+		return;
+  
     MlmeEnqueue(pAd, SYNC_STATE_MACHINE, MT2_SCAN_TIMEOUT, 0, NULL);
     RTUSBMlmeUp(pAd);
 }
@@ -212,7 +224,7 @@ VOID MlmeScanReqAction(
 		//
 		if (OPSTATUS_TEST_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED) && (INFRA_ON(pAd)))
 		{
-            NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+            NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
             if (NStatus == NDIS_STATUS_SUCCESS)
             {
 				pHdr80211 = (PHEADER_802_11) pOutBuffer;
@@ -848,7 +860,7 @@ VOID PeerBeacon(
 		
 		// if the ssid matched & bssid unmatched, we should select the bssid with large value.
 		// This might happened when two STA start at the same time
-		if ((! is_my_bssid) && ADHOC_ON(pAd))
+		if ((! is_my_bssid) && ADHOC_ON(pAd) && (BssType == BSS_ADHOC))
 		{
 			INT	i;
 
@@ -1038,7 +1050,8 @@ VOID PeerBeacon(
 #ifndef	SINGLE_ADHOC_LINKUP
 			// At least another peer in this IBSS, declare MediaState as CONNECTED
 			if (ADHOC_ON(pAd) && 
-				!OPSTATUS_TEST_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED))
+				!OPSTATUS_TEST_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED) &&
+				(BssType == BSS_ADHOC))
 			{
 				OPSTATUS_SET_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED);
 
@@ -1107,7 +1120,7 @@ VOID PeerProbeReqAction(
 #endif
 			
             // allocate and send out ProbeReq frame
-            NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+            NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
             if (NStatus != NDIS_STATUS_SUCCESS)
 	            return;
             
@@ -1261,7 +1274,7 @@ VOID ScanNextChannel(
 		//
 		if (OPSTATUS_TEST_FLAG(pAd, fOP_STATUS_MEDIA_STATE_CONNECTED) && (INFRA_ON(pAd)))
 		{
-            NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+            NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
             if (NStatus == NDIS_STATUS_SUCCESS)
             {
 				pHdr80211 = (PHEADER_802_11) pOutBuffer;
@@ -1343,7 +1356,7 @@ VOID ScanNextChannel(
 	
         if ((ScanType == SCAN_ACTIVE) || (ScanType == FAST_SCAN_ACTIVE))
         {
-            NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+            NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
             if (NStatus != NDIS_STATUS_SUCCESS)
             {
     	        DBGPRINT(RT_DEBUG_TRACE, "SYNC - ScanNextChannel() allocate memory fail\n");
@@ -1397,18 +1410,6 @@ VOID ScanNextChannel(
             	FrameLen += Tmp;
             }
 
-            // add Simple Config Information Element
-            if (pAd->PortCfg.bWscCapable && pAd->PortCfg.WscIEProbeReq.ValueLen)
-            {
-                ULONG WscTmpLen = 0;
-
-                MakeOutgoingFrame(pOutBuffer+FrameLen,                      &WscTmpLen,
-                                  pAd->PortCfg.WscIEProbeReq.ValueLen,      pAd->PortCfg.WscIEProbeReq.Value,
-                                  END_OF_ARGS);
-                FrameLen += WscTmpLen;
-            }
-
-
             if (pAd->PortCfg.bGetAPConfig) 
             {
                 UCHAR RalinkSpecificIEForGetCfg[6] = {IE_VENDOR_SPECIFIC, 4, 0x00, 0x0c, 0x43, 0x80};
@@ -1489,6 +1490,13 @@ VOID InvalidStateWhenStart(
 VOID EnqueuePsPoll(
     IN PRTMP_ADAPTER pAd) 
 {
+#ifdef RALINK_ATE
+    if(pAd->ate.Mode != ATE_STASTART)
+    {
+	return;
+    }
+#endif	
+
     DBGPRINT(RT_DEBUG_TRACE, "SYNC - send PsPoll ...\n");
     MiniportMMRequest(pAd, (PUCHAR)&pAd->PsPollFrame, sizeof(PSPOLL_FRAME));
 }
@@ -1523,7 +1531,7 @@ VOID EnqueueBeaconFrame(
 #endif
     DBGPRINT(RT_DEBUG_TRACE, "SYNC - driver sent BEACON (len=%d)...\n",pTxD->DataByteCnt);
 
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
 	if (NStatus != NDIS_STATUS_SUCCESS) 
 	{
 		DBGPRINT(RT_DEBUG_TRACE, "EnqueueBeaconFrame allocate memory fail\n");
@@ -1561,7 +1569,7 @@ VOID EnqueueProbeRequest(
     DBGPRINT(RT_DEBUG_TRACE, "force out a ProbeRequest ...\n");
     
 
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
     {
         DBGPRINT(RT_DEBUG_TRACE, "EnqueueProbeRequest() allocate memory fail\n");
@@ -1582,16 +1590,6 @@ VOID EnqueueProbeRequest(
                           pAd->ActiveCfg.SupRateLen,      pAd->ActiveCfg.SupRate, 
                           END_OF_ARGS);
 
-        // add Simple Config Information Element
-        if (pAd->PortCfg.bWscCapable && pAd->PortCfg.WscIEProbeReq.ValueLen) 
-        {
-            ULONG WscTmpLen = 0;
-
-            MakeOutgoingFrame(pOutBuffer+FrameLen,                   &WscTmpLen,
-        		              pAd->PortCfg.WscIEProbeReq.ValueLen,   pAd->PortCfg.WscIEProbeReq.Value,
-                              END_OF_ARGS);
-            FrameLen += WscTmpLen;
-        }
 
 	    if (pAd->PortCfg.bGetAPConfig)         
 	    {            
