@@ -68,6 +68,10 @@ extern UCHAR    CipherWpa2TemplateLen;
 #define     WPARSNIE    0xdd
 #define     WPA2RSNIE   0x30
 
+static UCHAR	prf_input[1024];
+
+#define ROUND_UP(__x, __y) \
+	(((unsigned long)((__x)+((__y)-1))) & ((unsigned long)~((__y)-1)))\
 
 /*
 	========================================================================
@@ -174,7 +178,6 @@ VOID    WpaEAPOLKeyAction(
 		tmpKeyinfo = SWAP16(tmpKeyinfo);
 		NdisMoveMemory(&pKeyDesc->KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }
-//	    *(USHORT *)((UCHAR *)pKeyDesc+1) = SWAP16(*(USHORT *)((UCHAR *)pKeyDesc+1));
 #endif
 
 	// Sanity check, this should only happen in WPA(2)-PSK mode
@@ -191,24 +194,23 @@ VOID    WpaEAPOLKeyAction(
 	DBGPRINT(RT_DEBUG_INFO, "KeyInfo EKD_DL %d\n", pKeyDesc->KeyInfo.EKD_DL);
 
 
-	// 1. Check EAPOL frame version and type
-    if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPAPSK)
-    {
-        if ((Elem->Msg[LENGTH_802_11+LENGTH_802_1_H] != EAPOL_VER) || (pKeyDesc->Type != WPA1_KEY_DESC))
-	    {
-		    DBGPRINT(RT_DEBUG_ERROR, "	 Key descripter	does not match with	WPA1 rule \n");
-		    return;
-	    }
-    }
-    else if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPA2PSK)
-    { 
-        // pass (Version != EAPOL_VER)
-        if (pKeyDesc->Type != WPA2_KEY_DESC)
-        {
-            DBGPRINT(RT_DEBUG_ERROR, "   Key descripter does not match with WPA2 rule \n");
-            return;
-        }
-    }
+	// 1. Check EAPOL frame type
+	if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPAPSK)
+	{
+		if (pKeyDesc->Type != WPA1_KEY_DESC)
+		{
+			DBGPRINT(RT_DEBUG_ERROR, "	 Key descripter	does not match with	WPA1 rule \n");
+			return;
+		}
+	}
+	else if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPA2PSK)
+	{
+		if (pKeyDesc->Type != WPA2_KEY_DESC)
+		{
+			DBGPRINT(RT_DEBUG_ERROR, "   Key descripter does not match with WPA2 rule \n");
+			return;
+		}
+	}
 
     // First validate replay counter, only accept message with larger replay counter
 	// Let equal pass, some AP start with all zero replay counter
@@ -271,7 +273,6 @@ VOID    WpaEAPOLKeyAction(
         tmpKeyinfo = SWAP16(tmpKeyinfo);
 	    NdisMoveMemory(&pKeyDesc->KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }    
-//	    *(USHORT *)((UCHAR *)pKeyDesc+1) = SWAP16(*(USHORT *)((UCHAR *)pKeyDesc+1));
 #endif
 
         // We will assume link is up (assoc suceess and port not secured).
@@ -378,7 +379,6 @@ VOID    WpaEAPOLKeyAction(
 	    tmpKeyinfo = SWAP16(tmpKeyinfo);
 	    NdisMoveMemory(&pKeyDesc->KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }    
-//	    *(USHORT *)((UCHAR *)pKeyDesc+1) = SWAP16(*(USHORT *)((UCHAR *)pKeyDesc+1));
 #endif
 	
 	    // We will assume link is up (assoc suceess and port not secured).
@@ -562,12 +562,11 @@ VOID	WpaPairMsg1Action(
     tmpKeyinfo = SWAP16(tmpKeyinfo);
     NdisMoveMemory(&Packet.KeyDesc.KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }      
-//	*(USHORT *)(&(Packet.KeyDesc.KeyInfo)) = SWAP16(*(USHORT *)(&(Packet.KeyDesc.KeyInfo)));
 #endif
 	
 	// Send EAPOL(0, 1, 0, 0, 0, K, 0, TSNonce, 0, MIC(TPTK), 0)
 	// Out buffer for transmitting message 2
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
 		return;					
 
@@ -603,7 +602,7 @@ VOID	WpaPairMsg1Action(
 	}
 	NdisMoveMemory(Packet.KeyDesc.KeyMic, Mic, LEN_KEY_DESC_MIC);
 
-    FrameLen = 0;
+	FrameLen = 0;
 	MakeOutgoingFrame(pOutBuffer,           &FrameLen,	
 	                  sizeof(HEADER_802_11),&Header_802_11,
 		              sizeof(EAPHEAD),      EAPHEAD, 
@@ -734,7 +733,7 @@ VOID    Wpa2PairMsg1Action(
 
     // Send EAPOL(0, 1, 0, 0, 0, K, 0, TSNonce, 0, MIC(TPTK), 0)
     // Out buffer for transmitting message 2        
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
         return;                 
 
@@ -830,7 +829,6 @@ VOID	WpaPairMsg3Action(
     tmpKeyinfo = SWAP16(tmpKeyinfo);
     NdisMoveMemory(&pMsg3->KeyDesc.KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }
-//	*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)) = SWAP16(*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)));
 #endif
 
 	// 1. Verify RSN IE & cipher type match
@@ -852,7 +850,6 @@ VOID	WpaPairMsg3Action(
     tmpKeyinfo = SWAP16(tmpKeyinfo);
     NdisMoveMemory(&pMsg3->KeyDesc.KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }
-//	*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)) = SWAP16(*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)));
 #endif
 	
 	// 2. Check MIC value
@@ -923,7 +920,6 @@ VOID	WpaPairMsg3Action(
     tmpKeyinfo = SWAP16(tmpKeyinfo);
     NdisMoveMemory(&pMsg3->KeyDesc.KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }
-//	*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)) = SWAP16(*(USHORT *)(&(pMsg3->KeyDesc.KeyInfo)));
 #endif
 	
 	// Key descriptor version and appropriate RSN IE
@@ -956,11 +952,10 @@ VOID	WpaPairMsg3Action(
     tmpKeyinfo = SWAP16(tmpKeyinfo);
     NdisMoveMemory(&Packet.KeyDesc.KeyInfo, &tmpKeyinfo, sizeof(USHORT));
 }
-//    *(USHORT *)&Packet.KeyDesc.KeyInfo = SWAP16(*(USHORT *)&Packet.KeyDesc.KeyInfo);
 #endif
 
 	// Out buffer for transmitting message 4
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
 		return;					
 
@@ -1036,15 +1031,31 @@ VOID    Wpa2PairMsg3Action(
     PEAPOL_PACKET       pMsg3;
     UCHAR               Mic[16], OldMic[16];    
     PNDIS_802_11_KEY    pPeerKey;
-    UCHAR               KEYDATA[512], Key[32];
+    UCHAR               *KEYDATA, Key[32];
     USHORT              NStatus;
+    UCHAR               *mpool, *DescKeyData;
 
     DBGPRINT(RT_DEBUG_TRACE, "Wpa2PairMsg3Action ----->\n");
     
+    mpool = kmalloc(1056, MEM_ALLOC_FLAG);  // allocate memory
+    if (mpool == NULL)
+        return;
+
+    // KEYDATA Len = 512
+    KEYDATA = (UCHAR *) ROUND_UP(mpool, 4);
+    // DescKeyData Len = 512
+    DescKeyData = (UCHAR *) ROUND_UP(KEYDATA+512, 4);
+    
     pHeader = (PHEADER_802_11) Elem->Msg;
     
     // Process message 3 frame.
     pMsg3 = (PEAPOL_PACKET) &Elem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
+    
+    // Store KeyData, protect received data more than MAX_LEN_OF_RSNIE
+    NdisZeroMemory(DescKeyData, 512);
+    NdisMoveMemory( DescKeyData, 
+                    &Elem->Msg[LENGTH_802_11 + LENGTH_802_1_H + sizeof(EAPOL_PACKET) - MAX_LEN_OF_RSNIE], 
+                    pMsg3->KeyDesc.KeyDataLen[1]);
 
 #ifdef BIG_ENDIAN
    // recovery original byte order, before forward Elem to another routine	   
@@ -1060,10 +1071,12 @@ VOID    Wpa2PairMsg3Action(
     // 1. Verify RSN IE & cipher type match
     if (pAd->PortCfg.WepStatus == Ndis802_11Encryption3Enabled && (pMsg3->KeyDesc.KeyInfo.KeyDescVer != 2))
     {
+        kfree(mpool);
         return;
     }
     else if (pAd->PortCfg.WepStatus == Ndis802_11Encryption2Enabled && (pMsg3->KeyDesc.KeyInfo.KeyDescVer != 1))
     {
+        kfree(mpool);
         return;
     }
 
@@ -1098,6 +1111,7 @@ VOID    Wpa2PairMsg3Action(
     if (!NdisEqualMemory(OldMic, Mic, LEN_KEY_DESC_MIC))
     {
         DBGPRINT(RT_DEBUG_ERROR, " MIC Different in msg 3 of 4-way handshake!!!!!!!!!! \n");
+        kfree(mpool);
         return;
     }
     else
@@ -1105,16 +1119,20 @@ VOID    Wpa2PairMsg3Action(
 
     // 3. Check Replay Counter, it has to be larger than last one. No need to be exact one larger
     if (RTMPCompareMemory(pMsg3->KeyDesc.ReplayCounter, pAd->PortCfg.ReplayCounter, LEN_KEY_DESC_REPLAY) != 1)
+    {
+        kfree(mpool);
         return;
-   
+    }
 
     // Update new replay counter
     NdisMoveMemory(pAd->PortCfg.ReplayCounter, pMsg3->KeyDesc.ReplayCounter, LEN_KEY_DESC_REPLAY);     
 
     // 4. Double check ANonce
     if (!NdisEqualMemory(pAd->PortCfg.ANonce, pMsg3->KeyDesc.KeyNonce, LEN_KEY_DESC_NONCE))
+    {
+        kfree(mpool);
         return;
-    
+    }
 
     // Obtain GTK
     // 5. Decrypt GTK from Key Data
@@ -1123,9 +1141,9 @@ VOID    Wpa2PairMsg3Action(
     {
     
         // Decrypt AES GTK  
-        AES_GTK_KEY_UNWRAP(&pAd->PortCfg.PTK[16], KEYDATA, pMsg3->KeyDesc.KeyDataLen[1],pMsg3->KeyDesc.KeyData);       
+        AES_GTK_KEY_UNWRAP(&pAd->PortCfg.PTK[16], KEYDATA, pMsg3->KeyDesc.KeyDataLen[1], DescKeyData);       
 
-        ParseKeyData(pAd, KEYDATA, pMsg3->KeyDesc.KeyDataLen[1]);
+        ParseKeyData(pAd, KEYDATA, pMsg3->KeyDesc.KeyDataLen[1], 0, FALSE);
     }
     else    // TKIP
     {
@@ -1139,7 +1157,7 @@ VOID    Wpa2PairMsg3Action(
         for (i = 0; i < 256; i++)
             ARCFOUR_BYTE(&pAd->PrivateInfo.WEPCONTEXT);
         // Decrypt GTK. Becareful, there is no ICV to check the result is correct or not
-        ARCFOUR_DECRYPT(&pAd->PrivateInfo.WEPCONTEXT, KEYDATA, pMsg3->KeyDesc.KeyData, pMsg3->KeyDesc.KeyDataLen[1]);       
+        ARCFOUR_DECRYPT(&pAd->PrivateInfo.WEPCONTEXT, KEYDATA, DescKeyData, pMsg3->KeyDesc.KeyDataLen[1]);       
 
         DBGPRINT_RAW(RT_DEBUG_TRACE, "KEYDATA = \n");
         for (i = 0; i < 100; i++)
@@ -1150,7 +1168,7 @@ VOID    Wpa2PairMsg3Action(
         }
         DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
 
-        ParseKeyData(pAd, KEYDATA, pMsg3->KeyDesc.KeyDataLen[1]);
+        ParseKeyData(pAd, KEYDATA, pMsg3->KeyDesc.KeyDataLen[1], 0, FALSE);
 
     }
     
@@ -1218,9 +1236,12 @@ VOID    Wpa2PairMsg3Action(
 #endif
 
     // Out buffer for transmitting message 4        
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
+    NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
     if (NStatus != NDIS_STATUS_SUCCESS)
+    {
+        kfree(mpool);
         return;                 
+    }
 
     // Prepare EAPOL frame for MIC calculation
     // Be careful, only EAPOL frame is counted for MIC calculation
@@ -1257,7 +1278,10 @@ VOID    Wpa2PairMsg3Action(
     // 7. Update PTK
     pPeerKey = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory
     if (pPeerKey == NULL)
+    {
+        kfree(mpool);
 		return;
+	}
 		
     NdisZeroMemory(pPeerKey, sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY);
     pPeerKey->Length    = sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY;
@@ -1275,6 +1299,7 @@ VOID    Wpa2PairMsg3Action(
     // Send using priority queue
     MiniportMMRequest(pAd, pOutBuffer, FrameLen);
 
+    kfree(mpool);
 
     DBGPRINT(RT_DEBUG_ERROR, "Wpa2PairMsg3Action <-----\n");
 
@@ -1307,38 +1332,45 @@ VOID	WpaGroupMsg1Action(
 	EAPOL_PACKET		Packet;
 	PEAPOL_PACKET		pGroup;
 	UCHAR				Mic[16], OldMic[16];
-	UCHAR				GTK[32], Key[32];
+	UCHAR				Key[32];
 	PNDIS_802_11_KEY	pGroupKey = NULL;
-//    UCHAR               MSG[MAX_LEN_OF_MLME_BUFFER];
 	UCHAR				*mpool, *MSG, *KEYDATA;
-    UCHAR               Header802_3[14];
-//    UCHAR               KEYDATA[512];	   
-    USHORT              NStatus;
+	UCHAR               Header802_3[14];   
+	USHORT              NStatus;
+	UCHAR               *DescKeyData;
 
- 
-	mpool = kmalloc(2*MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory
-    if (mpool == NULL)
-        return;
-		    
-	MSG = (UCHAR *) (((ULONG)(mpool+3)) & ((ULONG)~(0x03)));
-	KEYDATA =(UCHAR *) (((ULONG)(MSG+MAX_LEN_OF_MLME_BUFFER+3)) & ((ULONG)~(0x03)));
+	mpool = kmalloc(3104, MEM_ALLOC_FLAG);  // allocate memory
+	if (mpool == NULL)
+		return;
 
+	// MSG Len = 2048
+	MSG = (UCHAR *) ROUND_UP(mpool, 4);
+	// KEYDATA Len = 512
+	KEYDATA = (UCHAR *) ROUND_UP(MSG+2048, 4);
+	// DescKeyData Len = 512
+	DescKeyData = (UCHAR *) ROUND_UP(KEYDATA+512, 4);
 
-    if(NdisEqualMemory(&Elem->Msg[LENGTH_802_11 + 8], EAPHEAD, LENGTH_802_1_H))
-    {
-        DBGPRINT(RT_DEBUG_TRACE, "WpaGroupMsg1Action ----->MsgLen=%d\n",Elem->MsgLen);
-        NdisMoveMemory(MSG, Elem->Msg, LENGTH_802_11);
-        NdisMoveMemory(&MSG[LENGTH_802_11], &Elem->Msg[LENGTH_802_11+8], (Elem->MsgLen ));
-    }
-    else
-    {
-	    DBGPRINT(RT_DEBUG_TRACE, "WpaGroupMsg1Action ----->\n");
-        NdisMoveMemory(MSG, Elem->Msg, Elem->MsgLen);
-    }
+	if(NdisEqualMemory(&Elem->Msg[LENGTH_802_11 + 8], EAPHEAD, LENGTH_802_1_H))
+	{
+		DBGPRINT(RT_DEBUG_TRACE, "WpaGroupMsg1Action ----->MsgLen=%d\n",Elem->MsgLen);
+		NdisMoveMemory(MSG, Elem->Msg, LENGTH_802_11);
+		NdisMoveMemory(&MSG[LENGTH_802_11], &Elem->Msg[LENGTH_802_11+8], (Elem->MsgLen ));
+	}
+	else
+	{
+		DBGPRINT(RT_DEBUG_TRACE, "WpaGroupMsg1Action ----->\n");
+		NdisMoveMemory(MSG, Elem->Msg, Elem->MsgLen);
+	}
 	
 	// Process Group message 1 frame.
 	pGroup = (PEAPOL_PACKET) &MSG[LENGTH_802_11 + LENGTH_802_1_H];
 
+	// Store KeyData, protect received data more than MAX_LEN_OF_RSNIE
+	NdisZeroMemory(DescKeyData, 512);
+	NdisMoveMemory( DescKeyData, 
+                    &MSG[LENGTH_802_11 + LENGTH_802_1_H + sizeof(EAPOL_PACKET) - MAX_LEN_OF_RSNIE], 
+                    pGroup->KeyDesc.KeyDataLen[1]);
+
 #ifdef BIG_ENDIAN
    // recovery original byte order, before forward Elem to another routine	   
 {
@@ -1422,49 +1454,29 @@ VOID	WpaGroupMsg1Action(
 }
 #endif
 
-    // 3. Decrypt GTK from Key Data
-    DBGPRINT(RT_DEBUG_TRACE, " Install = %d!!!!EKD_DL = %d!!!!!KeyIndex = %d! \n", pGroup->KeyDesc.KeyInfo.Install, pGroup->KeyDesc.KeyInfo.EKD_DL,
+	// 3. Decrypt GTK from Key Data
+	DBGPRINT(RT_DEBUG_TRACE, " Install = %d!!!!EKD_DL = %d!!!!!KeyIndex = %d! \n", pGroup->KeyDesc.KeyInfo.Install, pGroup->KeyDesc.KeyInfo.EKD_DL,
         pGroup->KeyDesc.KeyInfo.KeyIndex);
 
   
-    pGroupKey = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory    
-    if (pGroupKey == NULL)
-    {
-        kfree(mpool);
-	    return;	
-    }
+	pGroupKey = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory    
+	if (pGroupKey == NULL)
+	{
+		kfree(mpool);
+		return;	
+	}
 			    
 	if (pAd->PortCfg.WepStatus	== Ndis802_11Encryption3Enabled)
 	{      
-		INT i;
-        // Decrypt AES GTK
-        NdisMoveMemory(KEYDATA, pGroup->KeyDesc.KeyData, 32);
+		// Decrypt AES GTK
+		NdisZeroMemory(KEYDATA, 128);
 	
-        //if (pGroup->KeyDesc.KeyInfo.EKD_DL == 1)
-            AES_GTK_KEY_UNWRAP(&pAd->PortCfg.PTK[16], KEYDATA,  pGroup->KeyDesc.KeyDataLen[1], pGroup->KeyDesc.KeyData);       
+		//if (pGroup->KeyDesc.KeyInfo.EKD_DL == 1)
+			AES_GTK_KEY_UNWRAP(&pAd->PortCfg.PTK[16], KEYDATA,  pGroup->KeyDesc.KeyDataLen[1], DescKeyData);       
 
 		// Update GTK
-        NdisZeroMemory(pGroupKey, sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY);
-	    pGroupKey->Length    = sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY;
-	    pGroupKey->KeyIndex  = 0x20000000 | pGroup->KeyDesc.KeyInfo.KeyIndex;
-	    pGroupKey->KeyLength = pGroup->KeyDesc.KeyLength[0] * 256 + pGroup->KeyDesc.KeyLength[1];
-
-	    COPY_MAC_ADDR(pGroupKey->BSSID, pAd->PortCfg.Bssid);
-	    NdisMoveMemory(pGroupKey->KeyMaterial, KEYDATA, 32);
-	    NdisMoveMemory(GTK, KEYDATA, 32);
-	    
-        DBGPRINT_RAW(RT_DEBUG_TRACE, "GTK = \n");
-        for (i = 0; i < 32; i++)
-        {
-            DBGPRINT_RAW(RT_DEBUG_TRACE, "%02x ", pGroup->KeyDesc.KeyData[i]);
-            if (i%16 == 15)
-                DBGPRINT_RAW(RT_DEBUG_TRACE, "\n ");
-        }
-        DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
-	    
+		ParseKeyData(pAd, KEYDATA, pGroup->KeyDesc.KeyDataLen[1], pGroup->KeyDesc.KeyInfo.KeyIndex, TRUE);
 	
-        // Call Add peer key function
-        RTMPWPAAddKeyProc(pAd, pGroupKey);
 	}
 	else	// TKIP
 	{
@@ -1475,29 +1487,28 @@ VOID	WpaGroupMsg1Action(
 		NdisMoveMemory(Key, pGroup->KeyDesc.KeyIv, 16);
 		NdisMoveMemory(&Key[16], &pAd->PortCfg.PTK[16], 16);
 		ARCFOUR_INIT(&pAd->PrivateInfo.WEPCONTEXT, Key, 32);
-        //discard first 256 bytes
+ 		//discard first 256 bytes
 		for (i = 0; i < 256; i++)
-            ARCFOUR_BYTE(&pAd->PrivateInfo.WEPCONTEXT);
+			ARCFOUR_BYTE(&pAd->PrivateInfo.WEPCONTEXT);
 		// Decrypt GTK. Becareful, there is no ICV to check the result is correct or not
-        ARCFOUR_DECRYPT(&pAd->PrivateInfo.WEPCONTEXT, GTK, pGroup->KeyDesc.KeyData, 32);
+		ARCFOUR_DECRYPT(&pAd->PrivateInfo.WEPCONTEXT, KEYDATA, DescKeyData, pGroup->KeyDesc.KeyDataLen[1]);
 
-        DBGPRINT_RAW(RT_DEBUG_TRACE, "GTK = \n");
-        for (i = 0; i < 32; i++)
-        {
-            DBGPRINT_RAW(RT_DEBUG_TRACE, "%2x ", GTK[i]);
-            if (i%16 == 15)
-                DBGPRINT_RAW(RT_DEBUG_TRACE, "\n ");
-        }
-        DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
-
-        //RTMPWPAAddKeyProc(pAd, pGroupKey);
-        //ParseKeyData(pAd, KEYDATA, pGroup->KeyDesc.KeyDataLen[1]);
+		DBGPRINT_RAW(RT_DEBUG_TRACE, "TKIP GTK = \n");
+		for (i = 0; i < 32; i++)
+		{
+			DBGPRINT_RAW(RT_DEBUG_TRACE, "%2x ", KEYDATA[i]);
+			if (i%16 == 15)
+			DBGPRINT_RAW(RT_DEBUG_TRACE, "\n ");
+		}
+		DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
 
+		// Update GTK
+		ParseKeyData(pAd, KEYDATA, pGroup->KeyDesc.KeyDataLen[1], pGroup->KeyDesc.KeyInfo.KeyIndex, TRUE);
 	}
 	
 	// 4. Construct Group Message 2
-    pAd->Sequence = ((pAd->Sequence) + 1) & (MAX_SEQ_NUMBER);
-    WPAMake8023Hdr(pAd, pAd->PortCfg.Bssid, Header802_3);
+	pAd->Sequence = ((pAd->Sequence) + 1) & (MAX_SEQ_NUMBER);
+	WPAMake8023Hdr(pAd, pAd->PortCfg.Bssid, Header802_3);
 
 	// Zero Group message 1 body
 	NdisZeroMemory(&Packet, sizeof(Packet));
@@ -1525,8 +1536,8 @@ VOID	WpaGroupMsg1Action(
 	Packet.KeyDesc.KeyInfo.KeyMic  = 1;
 
 	// Secure bit
-    if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPAPSK)
-	    Packet.KeyDesc.KeyInfo.Secure  = 1;
+	if (pAd->PortCfg.AuthMode == Ndis802_11AuthModeWPAPSK)
+		Packet.KeyDesc.KeyInfo.Secure  = 1;
 	
 	// Key Replay count	
 	NdisMoveMemory(Packet.KeyDesc.ReplayCounter, pGroup->KeyDesc.ReplayCounter, LEN_KEY_DESC_REPLAY);		
@@ -1543,12 +1554,13 @@ VOID	WpaGroupMsg1Action(
 #endif
 
 	// Out buffer for transmitting group message 2
-    NStatus = MlmeAllocateMemory(pAd, (PVOID)&pOutBuffer);  //Get an unused nonpaged memory
-    if (NStatus != NDIS_STATUS_SUCCESS) 
-    {
-        kfree(mpool);  
+	NStatus = MlmeAllocateMemory(pAd, (PVOID *)&pOutBuffer);  //Get an unused nonpaged memory
+	if (NStatus != NDIS_STATUS_SUCCESS) 
+	{
+		kfree(pGroupKey);
+		kfree(mpool);  
 		return;
-    }		
+	}		
 
 	// Prepare EAPOL frame for MIC calculation
 	// Be careful, only EAPOL frame is counted for MIC calculation
@@ -1588,23 +1600,12 @@ VOID	WpaGroupMsg1Action(
 		              END_OF_ARGS);
 
 	// 5. Copy frame to Tx ring and prepare for encryption
-    RTMPToWirelessSta(pAd, pOutBuffer, FrameLen);
+	RTMPToWirelessSta(pAd, pOutBuffer, FrameLen);
 	
-    // Free allocated memory
-    MlmeFreeMemory(pAd, pOutBuffer);
+	// Free allocated memory
+	MlmeFreeMemory(pAd, pOutBuffer);
 	
-	// 6. Update GTK
-	NdisZeroMemory(pGroupKey, sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY);
-	pGroupKey->Length    = sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY;
-	pGroupKey->KeyIndex  = 0x20000000 | pGroup->KeyDesc.KeyInfo.KeyIndex;
-	pGroupKey->KeyLength = pGroup->KeyDesc.KeyLength[0] * 256 + pGroup->KeyDesc.KeyLength[1];
-
-	COPY_MAC_ADDR(pGroupKey->BSSID, pAd->PortCfg.Bssid);
-	NdisMoveMemory(pGroupKey->KeyMaterial, GTK, LEN_EAP_KEY);
-	// Call Add peer key function
-	RTMPWPAAddKeyProc(pAd, pGroupKey);
-	
-    kfree(pGroupKey);
+	kfree(pGroupKey);
 	kfree(mpool);
     
 	DBGPRINT(RT_DEBUG_TRACE, "WpaGroupMsg1Action <-----\n");
@@ -1721,85 +1722,156 @@ VOID	HMAC_SHA1(
     ========================================================================
 */
 VOID ParseKeyData(
-    IN  PRTMP_ADAPTER   pAd,
-    IN  PUCHAR          pKeyData,
-    IN  UCHAR           KeyDataLen)
+	IN  PRTMP_ADAPTER   pAd,
+	IN  PUCHAR          pKeyData,
+	IN  UCHAR           KeyDataLen,
+	IN	UCHAR			KeyIdx,
+	IN	BOOLEAN			IsGroupMsg)
 {
-    PKDE_ENCAP          pKDE = NULL;
-    PNDIS_802_11_KEY    pGroupKey = NULL;
-    PUCHAR              pMyKeyData = pKeyData;
-    UCHAR               KeyDataLength = KeyDataLen;
-    UCHAR               GTKLEN;
-    INT                 i;
-    
-    if (!NdisEqualMemory(pKeyData, pAd->PortCfg.RSN_IE, pAd->PortCfg.RSN_IELen))
-    {
-        DBGPRINT(RT_DEBUG_ERROR, " RSN IE mismatched !!!!!!!!!! \n");
-    }
-    else
-        DBGPRINT(RT_DEBUG_TRACE, " RSN IE matched !!!!!!!!!! \n");
+	PKDE_ENCAP          pKDE = NULL;
+	PNDIS_802_11_KEY    pGroupKey = NULL;
+	PUCHAR              pMyKeyData=NULL;// = pKeyData;
+	UCHAR				*KeyDataBuf, *mpool;
+	UCHAR               KeyDataLength = KeyDataLen;
+	UCHAR               GTKLEN;
+	INT                 i;
+	ULONG               Idx;   
+	PUCHAR              pVIE=NULL;
+	UCHAR               Len;
+	PEID_STRUCT         pEid;
+	BOOLEAN				bKDE_Key = FALSE;	// Key Data Encapsulation format
+
+	mpool = kmalloc(256, MEM_ALLOC_FLAG);  // allocate memory
+	if (mpool == NULL)
+		return;
+
+	// KeyDataBuf len = 256.
+	KeyDataBuf = (UCHAR *) ROUND_UP(mpool, 4);
+	// Init the buffer
+	NdisZeroMemory(KeyDataBuf, 256);
+	
+	if((Idx = BssTableSearch(&pAd->ScanTab, pAd->PortCfg.Bssid, pAd->PortCfg.Channel)) == BSS_NOT_FOUND)
+	{
+		DBGPRINT(RT_DEBUG_ERROR, "%s, Can't find BSS\n", __FUNCTION__);
+		return;
+	}
+	
+	pVIE = pAd->ScanTab.BssEntry[Idx].VarIEs;
+	Len = (UCHAR)pAd->ScanTab.BssEntry[Idx].VarIELen;
+	while(Len > 0)
+	{
+	    pEid = (PEID_STRUCT) pVIE;	
+		if (pEid->Eid != IE_RSN)
+		{
+			pVIE += (pEid->Len + 2);
+			Len  -= (pEid->Len + 2);
+			continue;
+		}
+
+		if (!NdisEqualMemory(pKeyData, pEid->Octet, pEid->Len))
+		{
+//			DBGPRINT(RT_DEBUG_ERROR, " RSN IE mismatched !!!!!!!!!! \n");
+		}
+		else
+		{
+//			DBGPRINT(RT_DEBUG_TRACE, " RSN IE matched !!!!!!!!!! \n");
+		}
+        
+		Len  -= (pEid->Len + 2);    
+		break;
+	}
 
-    DBGPRINT(RT_DEBUG_ERROR, "KeyDataLen = %d  \n", KeyDataLen);
 
 /*
 ====================================================================
 ======================================================================
 */
-    if ((*pKeyData == WPARSNIE) && (*(pKeyData+1) != 0) && (KeyDataLength >= (2 + *(pKeyData+1))))      
-    {
-        pMyKeyData = pKeyData + *(pKeyData+1) + 2;
-        KeyDataLength -= (2 + *(pKeyData+1));
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"WPA RSN IE length %d contained in Msg3 = \n", (2 + *(pKeyData+1)));
-    }
-    if ((*pMyKeyData == WPA2RSNIE) && (*(pMyKeyData+1) != 0) && (KeyDataLength >= (2 + *(pMyKeyData+1))))       
-    {
-        pMyKeyData += (*(pMyKeyData+1) + 2);
-        KeyDataLength -= (2 + *(pMyKeyData+1));
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"WPA2 RSN IE length %d contained in Msg3 = \n", (2 + *(pMyKeyData+1)));
-    }
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"KeyDataLength %d   \n", KeyDataLength);
-    if ((KeyDataLength >= 8) && (KeyDataLength <= sizeof(KDE_ENCAP)))
-    {
-        pKDE = (PKDE_ENCAP) pMyKeyData;
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE = \n");
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->Type %x:", pKDE->Type);
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->Len 0x%x:", pKDE->Len);
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->OUI %x %x %x :", pKDE->OUI[0],pKDE->OUI[1],pKDE->OUI[2] );
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"\n");                      
-    }   
+	NdisMoveMemory(KeyDataBuf, pKeyData, KeyDataLen);
+	pMyKeyData =(PUCHAR) KeyDataBuf;
+	if (!IsGroupMsg)
+	{
+		do{
+			if ((*pMyKeyData == WPARSNIE) && (*(pMyKeyData+1) != 0) && (KeyDataLength >= (2 + *(pMyKeyData+1))))      
+			{
+				if ((*(pMyKeyData+2) == 0x00) && (*(pMyKeyData+3) == 0x0f) && (*(pMyKeyData+4) == 0xac))
+					break;
+					
+				DBGPRINT_RAW(RT_DEBUG_TRACE,"WPA WPA_IE length %d contained in this message \n", (2 + *(pMyKeyData+1)));
+				KeyDataLength -= (2 + *(pMyKeyData+1));
+				pMyKeyData += *(pMyKeyData+1) + 2;
+			}
+			if ((*pMyKeyData == WPA2RSNIE) && (*(pMyKeyData+1) != 0) && (KeyDataLength >= (2 + *(pMyKeyData+1))))       
+			{
+				DBGPRINT_RAW(RT_DEBUG_TRACE,"WPA2 RSN_IE length %d contained in this message \n", (2 + *(pMyKeyData+1)));
+				KeyDataLength -= (2 + *(pMyKeyData+1));
+				pMyKeyData += (*(pMyKeyData+1) + 2);
+			}
+			
+		} while(KeyDataLength > 0);
+	}	
+	DBGPRINT_RAW(RT_DEBUG_TRACE,"KeyDataLength %d   \n", KeyDataLength);
+
+	// GTK key Data Encapsulation
+	pKDE = (PKDE_ENCAP) pMyKeyData;
+	if((KeyDataLength >= 8) && (pKDE->Type == WPARSNIE) && (pKDE->OUI[0] == 0x00) && (pKDE->OUI[1]== 0x0f) && (pKDE->OUI[2]== 0x0ac) && (pKDE->DataType== 0x01))
+	{   
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE = \n");
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->Type %x:", pKDE->Type);
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->Len 0x%x:", pKDE->Len);
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"pKDE->OUI %x %x %x :", pKDE->OUI[0],pKDE->OUI[1],pKDE->OUI[2] );
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"\n");                      
+
+		bKDE_Key = TRUE;
+	}   
     
-    if (pKDE->GTKEncap.Kid == 0)
-    {
-        DBGPRINT_RAW(RT_DEBUG_ERROR,"GTK Key index zero , error\n");
-        return;
-    }
+	if ((bKDE_Key == TRUE) && (pKDE->GTKEncap.Kid == 0))
+	{
+		DBGPRINT_RAW(RT_DEBUG_ERROR,"GTK Key index zero , error\n");
+		return;
+	}
 
-        GTKLEN = pKDE->Len -6;
 
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"GTK Key[%d] len=%d ", pKDE->GTKEncap.Kid, GTKLEN);
-        for (i = 0; i < GTKLEN; i++)
-        {
-            DBGPRINT_RAW(RT_DEBUG_TRACE,"%02x:", pKDE->GTKEncap.GTK[i]);
-        }
-        DBGPRINT_RAW(RT_DEBUG_TRACE,"\n");                       
+	// Update GTK
+	pGroupKey = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory
+	if (pGroupKey == NULL)
+		return;
 
-        // Update GTK
-        pGroupKey = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  // allocate memory
-        if (pGroupKey == NULL)
-		    return;
-	    
-        NdisZeroMemory(pGroupKey, sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY);
-	    pGroupKey->Length    = sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY;
-	    pGroupKey->KeyIndex  = 0x20000000 | pKDE->GTKEncap.Kid;
-	    pGroupKey->KeyLength = GTKLEN;
+	NdisZeroMemory(pGroupKey, sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY);
+	pGroupKey->Length = sizeof(NDIS_802_11_KEY) + LEN_EAP_KEY;
+	COPY_MAC_ADDR(pGroupKey->BSSID, pAd->PortCfg.Bssid);
 
-	    COPY_MAC_ADDR(pGroupKey->BSSID, pAd->PortCfg.Bssid);
-	    NdisMoveMemory(pGroupKey->KeyMaterial, pKDE->GTKEncap.GTK, 32);
+	if (bKDE_Key)	// GTK Key Data Encapsulation
+	{
+		GTKLEN = pKDE->Len -6;
 
-        // Call Add peer key function
-        RTMPWPAAddKeyProc(pAd, pGroupKey);
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"GTK Key[%d] len=%d ", pKDE->GTKEncap.Kid, GTKLEN);
+		for (i = 0; i < GTKLEN; i++)
+		{
+			DBGPRINT_RAW(RT_DEBUG_TRACE,"%02x:", pKDE->GTKEncap.GTK[i]);
+		}
+		DBGPRINT_RAW(RT_DEBUG_TRACE,"\n");                       
 
-        kfree(pGroupKey);
+		pGroupKey->KeyIndex  = 0x20000000 | pKDE->GTKEncap.Kid;
+		pGroupKey->KeyLength = GTKLEN;
+
+		NdisMoveMemory(pGroupKey->KeyMaterial, pKDE->GTKEncap.GTK, 32);
+
+		// Call Add peer key function
+		RTMPWPAAddKeyProc(pAd, pGroupKey);
+	}		
+	else if (KeyIdx != 0)	// non-KDE Key
+	{
+		pGroupKey->KeyIndex  = 0x20000000 | KeyIdx;
+		pGroupKey->KeyLength = 32;	//KeyDataLen
+
+		NdisMoveMemory(pGroupKey->KeyMaterial, pMyKeyData, LEN_EAP_KEY);
+
+		// Call Add peer key function
+		RTMPWPAAddKeyProc(pAd, pGroupKey);
+	}
+
+	kfree(pGroupKey);
+	kfree(mpool);
 
 }
 
@@ -1842,10 +1914,17 @@ VOID	PRF(
 	IN	INT		len)
 {
 	INT		i;
-	UCHAR	input[1024];
+	UCHAR	*input;
 	INT		currentindex = 0;
-
 	INT		total_len;
+	BOOLEAN	bfree = TRUE;
+
+	input = kmalloc(1024, MEM_ALLOC_FLAG);  // allocate memory
+    if (input == NULL) 
+    {
+    	input = prf_input;
+		bfree = FALSE;
+    }
 	
 	NdisMoveMemory(input, prefix, prefix_len);
 	input[prefix_len] =	0;
@@ -1859,6 +1938,9 @@ VOID	PRF(
 		currentindex +=	20;
 		input[total_len - 1]++;
 	}	
+
+	if (bfree==TRUE) 
+	    kfree(input);
 }
 
 /*
@@ -1893,35 +1975,35 @@ VOID    WpaCountPTK(
 
 	NdisZeroMemory(temp, sizeof(temp));
 
-    // Get smaller address
-    if (RTMPCompareMemory(SA, AA, 6) == 1)
-	    NdisMoveMemory(concatenation, AA, 6);
-    else
-        NdisMoveMemory(concatenation, SA, 6);
-    CurrPos += 6;
+	// Get smaller address
+	if (RTMPCompareMemory(SA, AA, 6) == 1)
+		NdisMoveMemory(concatenation, AA, 6);
+	else
+		NdisMoveMemory(concatenation, SA, 6);
+	CurrPos += 6;
 
-    // Get larger address
-    if (RTMPCompareMemory(SA, AA, 6) == 1)
-	    NdisMoveMemory(&concatenation[CurrPos], SA, 6);
-    else
-	    NdisMoveMemory(&concatenation[CurrPos], AA, 6);
-    CurrPos += 6;
+	// Get larger address
+	if (RTMPCompareMemory(SA, AA, 6) == 1)
+		NdisMoveMemory(&concatenation[CurrPos], SA, 6);
+	else
+		NdisMoveMemory(&concatenation[CurrPos], AA, 6);
+	CurrPos += 6;
 
-    // Get smaller address
-    if (RTMPCompareMemory(ANonce, SNonce, 32) == 1)
-	    NdisMoveMemory(&concatenation[CurrPos], SNonce, 32);
-    else
-	    NdisMoveMemory(&concatenation[CurrPos], ANonce, 32);
-    CurrPos += 32;
+	// Get smaller address
+	if (RTMPCompareMemory(ANonce, SNonce, 32) == 1)
+		NdisMoveMemory(&concatenation[CurrPos], SNonce, 32);
+	else
+		NdisMoveMemory(&concatenation[CurrPos], ANonce, 32);
+	CurrPos += 32;
 
-    // Get larger address
-    if (RTMPCompareMemory(ANonce, SNonce, 32) == 1)
-	    NdisMoveMemory(&concatenation[CurrPos], ANonce, 32);
-    else
-	    NdisMoveMemory(&concatenation[CurrPos], SNonce, 32);
-    CurrPos += 32;
+	// Get larger address
+	if (RTMPCompareMemory(ANonce, SNonce, 32) == 1)
+		NdisMoveMemory(&concatenation[CurrPos], ANonce, 32);
+	else
+		NdisMoveMemory(&concatenation[CurrPos], SNonce, 32);
+	CurrPos += 32;
 
-    PRF(PMK, LEN_MASTER_KEY, Prefix, 22, concatenation, 76, output, len);
+	PRF(PMK, LEN_MASTER_KEY, Prefix, 22, concatenation, 76, output, len);
 
 }
 
@@ -1945,10 +2027,22 @@ VOID	GenRandom(
 	OUT	UCHAR			*random)
 {	
 	INT		i, curr;
-	UCHAR	local[80], KeyCounter[32];
-	UCHAR	result[80];
-	ULONG	CurrentTime;
+	UCHAR	        *local, *KeyCounter;
+	UCHAR	        *result;
+	unsigned long	CurrentTime;
 	UCHAR	prefix[] = {'I', 'n', 'i', 't', ' ', 'C', 'o', 'u', 'n', 't', 'e', 'r'};
+	UCHAR	        *mpool;
+
+	mpool = kmalloc(256, MEM_ALLOC_FLAG);  // allocate memory
+	if (mpool == NULL)
+        return;
+
+	// local Len = 80.
+	local = (UCHAR *) ROUND_UP(mpool, 4);
+	// KeyCounter Len = 32.
+	KeyCounter = (UCHAR *) ROUND_UP(local + 80, 4);
+	// result Len = 80.
+	result = (UCHAR *) ROUND_UP(KeyCounter + 32, 4);
 
 	NdisZeroMemory(result, 80);
 	NdisZeroMemory(local, 80);
@@ -1970,6 +2064,7 @@ VOID	GenRandom(
 		PRF(KeyCounter, 32, prefix,12, local,	curr, result, 32); 
 	}
 	NdisMoveMemory(random, result,	32);	
+	kfree(mpool);
 }
 
 /*
@@ -1983,8 +2078,7 @@ VOID	GenRandom(
     Return Value:
     
     Note:
-        This function references to RFC 3394 for aes key unwrap algorithm.
-            
+        This function references to RFC 3394 for aes key unwrap algorithm.            
     ========================================================================
 */
 VOID    AES_GTK_KEY_UNWRAP( 
@@ -1993,53 +2087,67 @@ VOID    AES_GTK_KEY_UNWRAP(
     IN UCHAR    c_len,
     IN  UCHAR   *ciphertext)
 {
-    UCHAR       A[8],   BIN[16], BOUT[16];
-    UCHAR       xor;
-    INT         i, j;
-    aes_context aesctx;
-    UCHAR       R[512];
-    INT         num_blocks = c_len/8;   // unit:64bits
+	UCHAR        *A, *BIN, *BOUT;
+	UCHAR       xor;
+	INT         i, j;
+	aes_context aesctx;
+	UCHAR        *R;
+	INT         num_blocks = c_len/8;   // unit:64bits
+	UCHAR       *mpool;
 
-    // Initialize
-    NdisMoveMemory(A, ciphertext, 8);
-    //Input plaintext
-    for (i = 0; i < (c_len-8); i++)
-    {
-        R[ i] = ciphertext[i + 8];
-    }
+	mpool = kmalloc(1024, MEM_ALLOC_FLAG);  // allocate memory
+	if (mpool == NULL)
+		return;
 
-    aes_set_key(&aesctx, key, 128);
+	// A len = 8.
+	A = (UCHAR *) ROUND_UP(mpool, 4);
+	// BIN len = 16.
+	BIN = (UCHAR *) ROUND_UP(A + 8, 4);
+	// BOUT len = 16.
+	BOUT = (UCHAR *) ROUND_UP(BIN + 16, 4);
+	// R len = 512.
+	R = (UCHAR *) ROUND_UP(BOUT + 16, 4);
 
-    for (j = 5; j >= 0; j--)
-    {
-        for(i = (num_blocks-1); i > 0; i--)
-        {
-            xor = (num_blocks -1 )* j + i;
-            NdisMoveMemory(BIN, A, 8);
-            BIN[7] = A[7] ^ xor;
-            NdisMoveMemory(&BIN[8], &R[(i-1)*8], 8);
-            aes_decrypt(&aesctx, BIN, BOUT);
-            NdisMoveMemory(A, &BOUT[0], 8);
-            NdisMoveMemory(&R[(i-1)*8], &BOUT[8], 8);
-        }
-    }
+	// Initialize
+	NdisMoveMemory(A, ciphertext, 8);
+	//Input plaintext
+	for (i = 0; i < (c_len-8); i++)
+	{
+		R[ i] = ciphertext[i + 8];
+	}
 
-    // OUTPUT
-    for (i = 0; i < c_len; i++)
-    {
-        plaintext[i] = R[i];
-    }
+	aes_set_key(&aesctx, key, 128);
 
-    DBGPRINT_RAW(RT_DEBUG_TRACE, "plaintext = \n");
-    for (i = 0; i < (num_blocks *8); i++)
-    {
-        DBGPRINT_RAW(RT_DEBUG_TRACE, "%2x ", plaintext[i]);
-        if (i%16 == 15)
-            DBGPRINT_RAW(RT_DEBUG_TRACE, "\n ");
-    }
-    DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
+	for (j = 5; j >= 0; j--)
+	{
+		for(i = (num_blocks-1); i > 0; i--)
+		{
+			xor = (num_blocks -1 )* j + i;
+			NdisMoveMemory(BIN, A, 8);
+			BIN[7] = A[7] ^ xor;
+			NdisMoveMemory(&BIN[8], &R[(i-1)*8], 8);
+			aes_decrypt(&aesctx, BIN, BOUT);
+			NdisMoveMemory(A, &BOUT[0], 8);
+			NdisMoveMemory(&R[(i-1)*8], &BOUT[8], 8);
+		}
+	}
 
-    
+	// OUTPUT
+	for (i = 0; i < c_len; i++)
+	{
+		plaintext[i] = R[i];
+	}
+
+	DBGPRINT_RAW(RT_DEBUG_TRACE, "plaintext = \n");
+	for (i = 0; i < (num_blocks *8); i++)
+	{
+		DBGPRINT_RAW(RT_DEBUG_TRACE, "%2x ", plaintext[i]);
+		if (i%16 == 15)
+			DBGPRINT_RAW(RT_DEBUG_TRACE, "\n ");
+	}
+	DBGPRINT_RAW(RT_DEBUG_TRACE, "\n  \n");
+
+	kfree(mpool);
 }
 
 /*
