@@ -66,7 +66,8 @@ static int ralink_set_oid(struct wpa_driver_ralink_data *drv, unsigned short oid
 	iwr.u.data.pointer = (caddr_t) buf;
 	iwr.u.data.length = len;
 
-	if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0) {
+	if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0)
+	{
 		wpa_printf(MSG_DEBUG, "%s: oid=0x%x len (%d) failed",
 			   __func__, oid, len);
 		free(buf);
@@ -87,7 +88,8 @@ static int wpa_driver_ralink_get_bssid(void *priv, u8 *bssid)
 	memset(&iwr, 0, sizeof(iwr));
 	strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
 
-	if (ioctl(drv->ioctl_sock, SIOCGIWAP, &iwr) < 0) {
+	if (ioctl(drv->ioctl_sock, SIOCGIWAP, &iwr) < 0)
+	{
 		perror("ioctl[SIOCGIWAP]");
 		ret = -1;
 	}
@@ -109,7 +111,6 @@ static int wpa_driver_ralink_get_ssid(void *priv, u8 *ssid)
 	int ret = 0;
 	BOOLEAN	ieee8021x_mode = FALSE;
 	BOOLEAN ieee8021x_required_key = FALSE;
-
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
 	memset(&iwr, 0, sizeof(iwr));
@@ -117,7 +118,8 @@ static int wpa_driver_ralink_get_ssid(void *priv, u8 *ssid)
 	iwr.u.essid.pointer = (caddr_t) ssid;
 	iwr.u.essid.length = 32;
 
-	if (ioctl(drv->ioctl_sock, SIOCGIWESSID, &iwr) < 0) {
+	if (ioctl(drv->ioctl_sock, SIOCGIWESSID, &iwr) < 0)
+	{
 		perror("ioctl[SIOCGIWESSID]");
 		ret = -1;
 	} else
@@ -133,13 +135,15 @@ static int wpa_driver_ralink_get_ssid(void *priv, u8 *ssid)
 	if(wpa_s->conf->ap_scan == 0) 		
 	{
 		// Read BSSID form driver 
-		if (wpa_driver_ralink_get_bssid(priv, bssid) < 0) {
+		if (wpa_driver_ralink_get_bssid(priv, bssid) < 0)
+		{
 			wpa_printf(MSG_WARNING, "Could not read BSSID from driver.");
 			return ret;
 		}
 						
 		entry = wpa_s->conf->ssid;
-		while (entry) {
+		while (entry)
+		{
 			if (!entry->disabled && ssid_len == entry->ssid_len && memcmp(ssid_str, entry->ssid, ssid_len) == 0 &&
 		    	(!entry->bssid_set || memcmp(bssid, entry->bssid, ETH_ALEN) == 0))
 			{		
@@ -211,10 +215,11 @@ static int wpa_driver_ralink_set_ssid(struct wpa_driver_ralink_data *drv,
 	iwr.u.data.pointer = (caddr_t) buf;
 	iwr.u.data.length = sizeof(NDIS_802_11_SSID);
        
-	   if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0) {
+	if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0)
+	{
 		perror("ioctl[RT_PRIV_IOCTL] -- OID_802_11_SSID");
 		ret = -1;
-           }
+	}
 	free(buf);	
 	return ret;
 }
@@ -228,7 +233,8 @@ static void wpa_driver_ralink_event_pmkid(struct wpa_driver_ralink_data *drv,
 
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
-	if (data_len < 8) {
+	if (data_len < 8)
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: Too short PMKID Candidate List "
 			   "Event (len=%d)", data_len);
 		return;
@@ -238,19 +244,25 @@ static void wpa_driver_ralink_event_pmkid(struct wpa_driver_ralink_data *drv,
 		   "NumCandidates %d",
 		   (int) pmkid->Version, (int) pmkid->NumCandidates);
 
-	if (pmkid->Version != 1) {
+	if (pmkid->Version != 1) 
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: Unsupported PMKID Candidate List "
 			   "Version %d", (int) pmkid->Version);
 		return;
 	}
 
-	if (data_len < 8 + pmkid->NumCandidates * sizeof(PMKID_CANDIDATE)) {
+	if (data_len < 8 + pmkid->NumCandidates * sizeof(PMKID_CANDIDATE)) 
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: PMKID Candidate List underflow");
+		
 		return;
 	}
 
+
+	
 	memset(&event, 0, sizeof(event));
-	for (i = 0; i < pmkid->NumCandidates; i++) {
+	for (i = 0; i < pmkid->NumCandidates; i++)
+	{
 		PMKID_CANDIDATE *p = &pmkid->CandidateList[i];
 		wpa_printf(MSG_DEBUG, "RALINK: %d: " MACSTR " Flags 0x%x",
 			   i, MAC2STR(p->BSSID), (int) p->Flags);
@@ -273,7 +285,8 @@ static int wpa_driver_ralink_set_pmkid(struct wpa_driver_ralink_data *drv)
 
 	count = 0;
 	entry = drv->pmkid;
-	while (entry) {
+	while (entry)
+	{
 		count++;
 		if (count >= drv->no_of_pmkid)
 			break;
@@ -287,12 +300,13 @@ static int wpa_driver_ralink_set_pmkid(struct wpa_driver_ralink_data *drv)
 	p->Length = len;
 	p->BSSIDInfoCount = count;
 	entry = drv->pmkid;
-	for (i = 0; i < count; i++) {
+	for (i = 0; i < count; i++)
+	{
 		memcpy(&p->BSSIDInfo[i].BSSID, entry->bssid, ETH_ALEN);
 		memcpy(&p->BSSIDInfo[i].PMKID, entry->pmkid, 16);
 		entry = entry->next;
 	}
-	wpa_hexdump(MSG_MSGDUMP, "NDIS: OID_802_11_PMKID", (char *) p, len);
+	wpa_hexdump(MSG_MSGDUMP, "NDIS: OID_802_11_PMKID", (const u8 *) p, len);
 	ret = ralink_set_oid(drv, OID_802_11_PMKID, (char *) p, len);
 	free(p);
 	return ret;
@@ -311,25 +325,31 @@ static int wpa_driver_ralink_add_pmkid(void *priv, const u8 *bssid,
 
 	prev = NULL;
 	entry = drv->pmkid;
-	while (entry) {
+	while (entry)
+	{
 		if (memcmp(entry->bssid, bssid, ETH_ALEN) == 0)
 			break;
 		prev = entry;
 		entry = entry->next;
 	}
 
-	if (entry) {
+	if (entry)
+	{
 		/* Replace existing entry for this BSSID and move it into the
 		 * beginning of the list. */
 		memcpy(entry->pmkid, pmkid, 16);
-		if (prev) {
+		if (prev)
+		{
 			prev->next = entry->next;
 			entry->next = drv->pmkid;
 			drv->pmkid = entry;
 		}
-	} else {
+	}
+	else
+	{
 		entry = malloc(sizeof(*entry));
-		if (entry) {
+		if (entry)
+		{
 			memcpy(entry->bssid, bssid, ETH_ALEN);
 			memcpy(entry->pmkid, pmkid, 16);
 			entry->next = drv->pmkid;
@@ -355,9 +375,11 @@ static int wpa_driver_ralink_remove_pmkid(void *priv, const u8 *bssid,
 	entry = drv->pmkid;
 	prev = NULL;
 	drv->pmkid = NULL;
-	while (entry) {
+	while (entry)
+	{
 		if (memcmp(entry->bssid, bssid, ETH_ALEN) == 0 &&
-		    memcmp(entry->pmkid, pmkid, 16) == 0) {
+		    memcmp(entry->pmkid, pmkid, 16) == 0)
+		{
 			if (prev)
 				prev->next = entry->next;
 			else
@@ -385,7 +407,8 @@ static int wpa_driver_ralink_flush_pmkid(void *priv)
 
 	pmkid = drv->pmkid;
 	drv->pmkid = NULL;
-	while (pmkid) {
+	while (pmkid)
+	{
 		prev = pmkid;
 		pmkid = pmkid->next;
 		free(prev);
@@ -395,7 +418,7 @@ static int wpa_driver_ralink_flush_pmkid(void *priv)
 	p.Length = 8;
 	p.BSSIDInfoCount = 0;
 	wpa_hexdump(MSG_MSGDUMP, "NDIS: OID_802_11_PMKID (flush)",
-		    (char *) &p, 8);
+		    (const u8 *) &p, 8);
 	return ralink_set_oid(drv, OID_802_11_PMKID, (char *) &p, 8);
 }
 
@@ -411,56 +434,66 @@ wpa_driver_ralink_event_wireless_custom(struct wpa_driver_ralink_data *drv,
 
 	memset(&data, 0, sizeof(data));
 	/* Host AP driver */
-	if (strncmp(custom, "MLME-MICHAELMICFAILURE.indication", 33) == 0) {
+	if (strncmp(custom, "MLME-MICHAELMICFAILURE.indication", 33) == 0)
+	{
+	    //receive a MICFAILURE report
 		data.michael_mic_failure.unicast =
 			strstr(custom, " unicast ") != NULL;
 		/* TODO: parse parameters(?) */
 		wpa_supplicant_event(ctx, EVENT_MICHAEL_MIC_FAILURE, &data);
-        }
-	else if (strncmp(custom, "ASSOCINFO(ReqIEs=", 17) == 0) {
+	}
+	else if (strncmp(custom, "ASSOCINFO_ReqIEs=", 17) == 0)
+	{
+        //receive assoc. req. IEs
 		char *spos;
 		int bytes;
 
 		spos = custom + 17;
-
-		bytes = strspn(spos, "0123456789abcdefABCDEF");
-		if (!bytes || (bytes & 1))
+        //get IE's length
+		bytes = strlen(spos);
+		if (!bytes)
 			return;
-		bytes /= 2;
+		
 
 		data.assoc_info.req_ies = malloc(bytes);
 		if (data.assoc_info.req_ies == NULL)
 			return;
 
 		data.assoc_info.req_ies_len = bytes;
-		hexstr2bin(spos, data.assoc_info.req_ies, bytes);
+		memcpy(data.assoc_info.req_ies, spos,  bytes);
 
-		spos += bytes * 2;
+        //skip the '\0' byte
+		spos += bytes + 1;
 
 		data.assoc_info.resp_ies = NULL;
 		data.assoc_info.resp_ies_len = 0;
 
-		if (strncmp(spos, " RespIEs=", 9) == 0) {
+		if (strncmp(spos, " RespIEs=", 9) == 0)
+		{
+            //receive assoc. resp. IEs
 			spos += 9;
-
-			bytes = strspn(spos, "0123456789abcdefABCDEF");
-			if (!bytes || (bytes & 1))
+            //get IE's length
+			bytes = strlen(spos);
+			if (!bytes)
 				goto done;
-			bytes /= 2;
+			
 
 			data.assoc_info.resp_ies = malloc(bytes);
 			if (data.assoc_info.resp_ies == NULL)
 				goto done;
 
 			data.assoc_info.resp_ies_len = bytes;
-			hexstr2bin(spos, data.assoc_info.resp_ies, bytes);
+			memcpy(data.assoc_info.resp_ies, spos,  bytes);
 		}
 
 		wpa_supplicant_event(ctx, EVENT_ASSOCINFO, &data);
 
+      //free allocated memory
 	done:
-		free(data.assoc_info.resp_ies);
-		free(data.assoc_info.req_ies);
+		if(data.assoc_info.resp_ies != NULL)
+			free(data.assoc_info.resp_ies);
+		if(data.assoc_info.req_ies != NULL)
+			free(data.assoc_info.req_ies);
 	}
 }
 
@@ -478,7 +511,8 @@ static void wpa_driver_ralink_event_wireless(struct wpa_driver_ralink_data *drv,
 	pos = data;
 	end = data + len;
 
-	while (pos + IW_EV_LCP_LEN <= end) {
+	while (pos + IW_EV_LCP_LEN <= end)
+	{
 		/* Event data may be unaligned, so make a local, aligned copy
 		 * before processing. */
 		memcpy(&iwe_buf, pos, IW_EV_LCP_LEN);
@@ -490,41 +524,48 @@ static void wpa_driver_ralink_event_wireless(struct wpa_driver_ralink_data *drv,
 		custom = pos + IW_EV_POINT_LEN;
 		
 		if (drv->we_version_compiled > 18 &&
-		    (iwe->cmd == IWEVCUSTOM)) {
+		    (iwe->cmd == IWEVCUSTOM))
+		{
 			/* WE-19 removed the pointer from struct iw_point */
 			char *dpos = (char *) &iwe_buf.u.data.length;
 			int dlen = dpos - (char *) &iwe_buf;
 			memcpy(dpos, pos + IW_EV_LCP_LEN,
 			       sizeof(struct iw_event) - dlen);
-		} else {
+		}
+		else
+		{
 			memcpy(&iwe_buf, pos, sizeof(struct iw_event));
 		    custom += IW_EV_POINT_OFF;
         }
 
-		switch (iwe->cmd) {
+		switch (iwe->cmd)
+		{
 		
-		case IWEVCUSTOM:
-			if (custom + iwe->u.data.length > end)
-				return;
-			buf = malloc(iwe->u.data.length + 1);
-			if (buf == NULL)
-				return;
-			memcpy(buf, custom, iwe->u.data.length);
-			buf[iwe->u.data.length] = '\0';
+			case IWEVCUSTOM:
+				if (custom + iwe->u.data.length > end)
+					return;
+				buf = malloc(iwe->u.data.length + 1);
+				if (buf == NULL)
+					return;
+				memcpy(buf, custom, iwe->u.data.length);
+				buf[iwe->u.data.length] = '\0';
 
-            if (wpa_s->conf->ap_scan == 1) {
-			   if ((iwe->u.data.flags == RT_ASSOC_EVENT_FLAG) || (iwe->u.data.flags == RT_REQIE_EVENT_FLAG) ||
-			       (iwe->u.data.flags == RT_RESPIE_EVENT_FLAG) || (iwe->u.data.flags == RT_ASSOCINFO_EVENT_FLAG)) {
-					   if (scanning_done == 0) {
-					      free(buf);
-						  return;
-					   }
-				   }
-			}
+        		if (wpa_s->conf->ap_scan == 1)
+				{
+			   		if ((iwe->u.data.flags == RT_ASSOC_EVENT_FLAG) || (iwe->u.data.flags == RT_REQIE_EVENT_FLAG) ||
+			       		(iwe->u.data.flags == RT_RESPIE_EVENT_FLAG) || (iwe->u.data.flags == RT_ASSOCINFO_EVENT_FLAG))
+			       	{
+						if (scanning_done == 0)
+						{
+							free(buf);
+							return;
+						}
+					}
+				}
 
-            if (iwe->u.data.flags == RT_ASSOC_EVENT_FLAG) {
+        if (iwe->u.data.flags == RT_ASSOC_EVENT_FLAG)
+		{
 	           wpa_printf(MSG_DEBUG, "Custom wireless event: receive ASSOCIATED_EVENT !!!");
-
 			   // determine whether the dynamic-WEP is used or not 
 			   if(wpa_s && wpa_s->current_ssid && wpa_s->current_ssid->key_mgmt == WPA_KEY_MGMT_IEEE8021X_NO_WPA)
 			   {
@@ -547,8 +588,9 @@ static void wpa_driver_ralink_event_wireless(struct wpa_driver_ralink_data *drv,
 			   }
 
                wpa_supplicant_event(ctx, EVENT_ASSOC, NULL);
-            }
-			else if (iwe->u.data.flags == RT_REQIE_EVENT_FLAG) {
+		}
+		else if (iwe->u.data.flags == RT_REQIE_EVENT_FLAG)
+		{
 			   wpa_printf(MSG_DEBUG, "Custom wireless event: receive ReqIEs !!!");
 			   drv->assoc_req_ies = malloc(iwe->u.data.length);
 
@@ -557,23 +599,36 @@ static void wpa_driver_ralink_event_wireless(struct wpa_driver_ralink_data *drv,
 
                drv->assoc_req_ies_len = iwe->u.data.length;
 			   memcpy(drv->assoc_req_ies, custom, iwe->u.data.length);
-			}
-			else if (iwe->u.data.flags == RT_RESPIE_EVENT_FLAG) {
+		}
+		else if (iwe->u.data.flags == RT_RESPIE_EVENT_FLAG)
+		{
 			   wpa_printf(MSG_DEBUG, "Custom wireless event: receive RespIEs !!!");
 			   drv->assoc_resp_ies = malloc(iwe->u.data.length);
 
 			   if (drv->assoc_resp_ies == NULL)
+			{
+				if (drv->assoc_req_ies != NULL)
+			    	free(drv->assoc_req_ies);
 			      return;
+			}
 
                drv->assoc_resp_ies_len = iwe->u.data.length;
 			   memcpy(drv->assoc_resp_ies, custom, iwe->u.data.length);
-			}
-			else if (iwe->u.data.flags == RT_ASSOCINFO_EVENT_FLAG) {
+		}
+		else if (iwe->u.data.flags == RT_ASSOCINFO_EVENT_FLAG)
+		{
 			   wpa_printf(MSG_DEBUG, "Custom wireless event: receive ASSOCINFO_EVENT !!!");
 			   assoc_info_buf = malloc(drv->assoc_req_ies_len + drv->assoc_resp_ies_len + 1);
 
 			   if (assoc_info_buf == NULL)
+			{
+				if (drv->assoc_req_ies != NULL)
+			       	free(drv->assoc_req_ies);
+			      	if (drv->assoc_resp_ies != NULL)
+			       	free(drv->assoc_resp_ies);
+			    	free(buf);
 			      return;
+			}
 
 			   memcpy(assoc_info_buf, drv->assoc_req_ies, drv->assoc_req_ies_len);
 			   info_pos = assoc_info_buf + drv->assoc_req_ies_len;
@@ -582,19 +637,22 @@ static void wpa_driver_ralink_event_wireless(struct wpa_driver_ralink_data *drv,
 			   wpa_driver_ralink_event_wireless_custom(drv, ctx, assoc_info_buf);
 			   free(drv->assoc_req_ies);
 			   free(drv->assoc_resp_ies);
+			       free(assoc_info_buf);
 			}
-		    else if (iwe->u.data.flags == RT_DISASSOC_EVENT_FLAG) {
+		    else if (iwe->u.data.flags == RT_DISASSOC_EVENT_FLAG)
+			{
 			   wpa_printf(MSG_DEBUG, "Custom wireless event: receive DISASSOCIATED_EVENT !!!");
                wpa_supplicant_event(ctx, EVENT_DISASSOC, NULL);
 			}
-			else if (iwe->u.data.flags == RT_PMKIDCAND_FLAG) {
+			else if (iwe->u.data.flags == RT_PMKIDCAND_FLAG)
+			{
 			   wpa_printf(MSG_DEBUG, "Custom wireless event: receive PMKIDCAND_EVENT !!!");
-               wpa_driver_ralink_event_pmkid(drv, custom, iwe->u.data.length);
+                            wpa_driver_ralink_event_pmkid(drv, (const u8 *)custom, iwe->u.data.length);
 			}
-            else {
+            else
+			{
 			    wpa_driver_ralink_event_wireless_custom(drv, ctx, buf);
             }
-			free(assoc_info_buf);
 			free(buf);
 			break;
 		
@@ -618,8 +676,7 @@ static void wpa_driver_ralink_event_rtm_newlink(struct wpa_driver_ralink_data *d
 		return;
 
 	ifi = NLMSG_DATA(h);
-       wpa_hexdump(MSG_DEBUG, "ifi: ",
-			    (u8 *)ifi, sizeof(struct ifinfomsg));
+    wpa_hexdump(MSG_DEBUG, "ifi: ", (u8 *)ifi, sizeof(struct ifinfomsg));
 	
 	nlmsg_len = NLMSG_ALIGN(sizeof(struct ifinfomsg));
        
@@ -629,21 +686,18 @@ static void wpa_driver_ralink_event_rtm_newlink(struct wpa_driver_ralink_data *d
 		return;
 
 	attr = (struct rtattr *) (((char *) ifi) + nlmsg_len);
-       wpa_hexdump(MSG_DEBUG, "attr1: ",
-			    (u8 *)attr,sizeof(struct rtattr));
+    wpa_hexdump(MSG_DEBUG, "attr1: ", (u8 *)attr,sizeof(struct rtattr));
 	rta_len = RTA_ALIGN(sizeof(struct rtattr));
-	wpa_hexdump(MSG_DEBUG, "attr2: ",
-			    (u8 *)attr,rta_len);
-	while (RTA_OK(attr, attrlen)) {
+	wpa_hexdump(MSG_DEBUG, "attr2: ", (u8 *)attr,rta_len);
+	while (RTA_OK(attr, attrlen))
+	{
 		printf("rta_type=%02x\n",attr->rta_type);
-		if (attr->rta_type == IFLA_WIRELESS) {
-			wpa_driver_ralink_event_wireless(
-				drv, ctx, ((char *) attr) + rta_len,
-				attr->rta_len - rta_len);
+		if (attr->rta_type == IFLA_WIRELESS)
+		{
+			wpa_driver_ralink_event_wireless(drv, ctx, ((char *) attr) + rta_len, attr->rta_len - rta_len);
 		} 
 		attr = RTA_NEXT(attr, attrlen);
-		wpa_hexdump(MSG_DEBUG, "attr3: ",
-			    (u8 *)attr,sizeof(struct rtattr));
+		wpa_hexdump(MSG_DEBUG, "attr3: ", (u8 *)attr,sizeof(struct rtattr));
 	}
 }
 
@@ -659,33 +713,35 @@ static void wpa_driver_ralink_event_receive(int sock, void *ctx,
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
 	fromlen = sizeof(from);
-	left = recvfrom(sock, buf, sizeof(buf), MSG_DONTWAIT,
-			(struct sockaddr *) &from, &fromlen);
-	if (left < 0) {
+	left = recvfrom(sock, buf, sizeof(buf), MSG_DONTWAIT, (struct sockaddr *) &from, &fromlen);
+
+	if (left < 0)
+	{
 		if (errno != EINTR && errno != EAGAIN)
 			perror("recvfrom(netlink)");
 		return;
 	}
 
 	h = (struct nlmsghdr *) buf;
-	wpa_hexdump(MSG_DEBUG, "h: ",
-			    (u8 *)h, h->nlmsg_len);
-	while (left >= sizeof(*h)) {
+	wpa_hexdump(MSG_DEBUG, "h: ", (u8 *)h, h->nlmsg_len);
+
+	while (left >= sizeof(*h))
+	{
 		int len, plen;
 
 		len = h->nlmsg_len;
 		plen = len - sizeof(*h);
-		if (len > left || plen < 0) {
+		if (len > left || plen < 0)
+		{
 			wpa_printf(MSG_DEBUG, "Malformed netlink message: "
-				   "len=%d left=%d plen=%d",
-				   len, left, plen);
+				"len=%d left=%d plen=%d", len, left, plen);
 			break;
 		}
 
-		switch (h->nlmsg_type) {
+		switch (h->nlmsg_type)
+		{
 		case RTM_NEWLINK:
-			wpa_driver_ralink_event_rtm_newlink(ctx, sock_ctx,
-							  h, plen);
+			wpa_driver_ralink_event_rtm_newlink(ctx, sock_ctx, h, plen);
 			break;
 		}
 
@@ -694,7 +750,8 @@ static void wpa_driver_ralink_event_receive(int sock, void *ctx,
 		h = (struct nlmsghdr *) ((char *) h + len);
 	}
 
-	if (left > 0) {
+	if (left > 0)
+	{
 		wpa_printf(MSG_DEBUG, "%d extra bytes in the end of netlink "
 			   "message", left);
 	}
@@ -712,9 +769,9 @@ ralink_get_we_version_compiled(struct wpa_driver_ralink_data *drv)
 	iwr.u.data.pointer = (caddr_t) &we_version_compiled;
 	iwr.u.data.flags = RT_OID_WE_VERSION_COMPILED;
 
-    if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0) {
-        wpa_printf(MSG_DEBUG, "%s:  failed",
-		    	   __func__);
+    if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0)
+	{
+		wpa_printf(MSG_DEBUG, "%s:  failed", __func__);
 		return -1;
 	}
 	
@@ -737,7 +794,8 @@ ralink_set_iface_flags(void *priv, int dev_up)
 	memset(&ifr, 0, sizeof(ifr));
 	snprintf(ifr.ifr_name, IFNAMSIZ, "%s", drv->ifname);
 
-	if (ioctl(drv->ioctl_sock, SIOCGIFFLAGS, &ifr) != 0) {
+	if (ioctl(drv->ioctl_sock, SIOCGIFFLAGS, &ifr) != 0)
+	{
 		perror("ioctl[SIOCGIFFLAGS]");
 		return -1;
 	}
@@ -747,7 +805,8 @@ ralink_set_iface_flags(void *priv, int dev_up)
 	else
 		ifr.ifr_flags &= ~IFF_UP;
 
-	if (ioctl(drv->ioctl_sock, SIOCSIFFLAGS, &ifr) != 0) {
+	if (ioctl(drv->ioctl_sock, SIOCSIFFLAGS, &ifr) != 0)
+	{
 		perror("ioctl[SIOCSIFFLAGS]");
 		return -1;
 	}
@@ -760,28 +819,33 @@ static void * wpa_driver_ralink_init(void *ctx, const char *ifname)
 	int s;
 	struct wpa_driver_ralink_data *drv;
 	struct ifreq ifr;
-        struct sockaddr_nl local;
-        struct wpa_supplicant *wpa_s;
+	struct sockaddr_nl local;
     BOOLEAN  enable_wpa_supplicant;
+    struct wpa_supplicant *wpa_s = NULL;
 
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
 	/* open socket to kernel */
-	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
+	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+	{
 		perror("socket");
 		return NULL;
 	}
 	/* do it */
 	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
-	if (ioctl(s, SIOCGIFINDEX, &ifr) < 0) {
+
+	if (ioctl(s, SIOCGIFINDEX, &ifr) < 0)
+	{
 		perror(ifr.ifr_name);
 		return NULL;
 	}
 
 
 	drv = malloc(sizeof(*drv));
+
 	if (drv == NULL)
 		return NULL;
+
 	memset(drv, 0, sizeof(*drv));
 	drv->ctx = ctx;
 	strncpy(drv->ifname, ifname, sizeof(drv->ifname));
@@ -790,7 +854,9 @@ static void * wpa_driver_ralink_init(void *ctx, const char *ifname)
 	//use netlink like this
 
 	s = socket(PF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
-	if (s < 0) {
+
+	if (s < 0)
+	{
 		perror("socket(PF_NETLINK,SOCK_RAW,NETLINK_ROUTE)");
 		close(drv->ioctl_sock);
 		free(drv);
@@ -800,7 +866,9 @@ static void * wpa_driver_ralink_init(void *ctx, const char *ifname)
 	memset(&local, 0, sizeof(local));
 	local.nl_family = AF_NETLINK;
 	local.nl_groups = RTMGRP_LINK;
-	if (bind(s, (struct sockaddr *) &local, sizeof(local)) < 0) {
+
+	if (bind(s, (struct sockaddr *) &local, sizeof(local)) < 0)
+	{
 		perror("bind(netlink)");
 		close(s);
 		close(drv->ioctl_sock);
@@ -826,6 +894,7 @@ static void * wpa_driver_ralink_init(void *ctx, const char *ifname)
 	}
 
     wpa_s = drv->ctx;
+
     if (wpa_s->conf->ap_scan == 1)
        scanning_done = 0;
 
@@ -840,6 +909,7 @@ static void wpa_driver_ralink_deinit(void *priv)
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
     enable_wpa_supplicant = FALSE;
+
     // trigger driver disable wpa_supplicant support
 	if (ralink_set_oid(drv, RT_OID_WPA_SUPPLICANT_SUPPORT, (char *) &enable_wpa_supplicant, sizeof(BOOLEAN)) < 0)
 	{
@@ -867,6 +937,7 @@ static void wpa_driver_ralink_scan_timeout(void *eloop_ctx,
 	wpa_supplicant_event(timeout_ctx, EVENT_SCAN_RESULTS, NULL);
 
 	scanning_done = 1;
+
 }
 
 static int wpa_driver_ralink_scan(void *priv, const u8 *ssid,
@@ -884,18 +955,20 @@ static int wpa_driver_ralink_scan(void *priv, const u8 *ssid,
 		return -1;
 	}
 
+	wpa_driver_ralink_set_ssid(drv, ssid, ssid_len);
+
 	memset(&iwr, 0, sizeof(iwr));
 	strncpy(iwr.ifr_name, drv->ifname, IFNAMSIZ);
 
-	if (ioctl(drv->ioctl_sock, SIOCSIWSCAN, &iwr) < 0) {
+	if (ioctl(drv->ioctl_sock, SIOCSIWSCAN, &iwr) < 0)
+	{
 		perror("ioctl[SIOCSIWSCAN]");
 		ret = -1;
 	}
 
 	/* Not all drivers generate "scan completed" wireless event, so try to
 	 * read results after a timeout. */
-	eloop_register_timeout(4, 0, wpa_driver_ralink_scan_timeout, drv,
-			       drv->ctx);
+	eloop_register_timeout(4, 0, wpa_driver_ralink_scan_timeout, drv, drv->ctx);
 
     scanning_done = 0;
 
@@ -930,14 +1003,16 @@ wpa_driver_ralink_get_scan_results(void *priv,
 	iwr.u.data.pointer = (caddr_t) buf;
 	iwr.u.data.flags = OID_802_11_BSSID_LIST;
 
-    if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0) {
+    if (ioctl(drv->ioctl_sock, RT_PRIV_IOCTL, &iwr) < 0)
+	{
 		free(buf);
 		return -1;
 	}
 
     memset(results, 0, max_size * sizeof(struct wpa_scan_result));
 
-    for (ap_num = 0, wbi = wsr->Bssid; ap_num < wsr->NumberOfItems; ++ap_num) {
+    for (ap_num = 0, wbi = wsr->Bssid; ap_num < wsr->NumberOfItems; ++ap_num)
+	{
 		memcpy(results[ap_num].bssid, &wbi->MacAddress, ETH_ALEN);
 		memcpy(results[ap_num].ssid, wbi->Ssid.Ssid, wbi->Ssid.SsidLength);
 		results[ap_num].ssid_len = wbi->Ssid.SsidLength;
@@ -957,18 +1032,24 @@ wpa_driver_ralink_get_scan_results(void *priv,
 		memcpy(&results[ap_num].caps, pos, 2);
 		pos += 2;
 		
-	    while (pos + 1 < end && pos + 2 + pos[1] <= end) {
+	   	while (pos + 1 < end && pos + 2 + pos[1] <= end)
+		{
 		    u8 ielen = 2 + pos[1];
-		    if (ielen > SSID_MAX_WPA_IE_LEN) {
+
+	    	if (ielen > SSID_MAX_WPA_IE_LEN)
+			{
 			    pos += ielen;
 			    continue;
 		    }
 
 		    if (pos[0] == GENERIC_INFO_ELEM && pos[1] >= 4 &&
-		        memcmp(pos + 2, "\x00\x50\xf2\x01", 4) == 0) {
+		       	memcmp(pos + 2, "\x00\x50\xf2\x01", 4) == 0)
+			{
 			    memcpy(results[ap_num].wpa_ie, pos, ielen);
 			    results[ap_num].wpa_ie_len = ielen;
-		    } else if (pos[0] == RSN_INFO_ELEM) {
+		    }
+			else if (pos[0] == RSN_INFO_ELEM)
+			{
 			    memcpy(results[ap_num].rsn_ie, pos, ielen);
 			    results[ap_num].rsn_ie_len = ielen;
 		    }
@@ -989,7 +1070,8 @@ static int ralink_set_auth_mode(struct wpa_driver_ralink_data *drv, NDIS_802_11_
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
 	if (ralink_set_oid(drv, OID_802_11_AUTHENTICATION_MODE,
-			 (char *) &auth_mode, sizeof(auth_mode)) < 0) {
+			 (char *) &auth_mode, sizeof(auth_mode)) < 0)
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: Failed to set "
 			   "OID_802_11_AUTHENTICATION_MODE (%d)",
 			   (int) auth_mode);
@@ -1012,8 +1094,10 @@ static int wpa_driver_ralink_remove_key(struct wpa_driver_ralink_data *drv,
 
 	rkey.Length = sizeof(rkey);
 	rkey.KeyIndex = key_idx;
+
 	if (pairwise)
 		rkey.KeyIndex |= 1 << 30;
+
 	memcpy(rkey.BSSID, bssid, ETH_ALEN);
 
 	res = ralink_set_oid(drv, OID_802_11_REMOVE_KEY, (char *) &rkey,
@@ -1024,7 +1108,8 @@ static int wpa_driver_ralink_remove_key(struct wpa_driver_ralink_data *drv,
 	{
 		res2 = ralink_set_oid(drv, OID_802_11_REMOVE_WEP,
 				    (char *) &index, sizeof(index));
-	} else
+	}
+	else
 		res2 = 0;
 
 	if (res < 0 && res2 < 0)
@@ -1044,11 +1129,14 @@ static int wpa_driver_ralink_add_wep(struct wpa_driver_ralink_data *drv,
 
 	len = 12 + key_len;
 	wep = malloc(len);
+
 	if (wep == NULL)
 		return -1;
+
 	memset(wep, 0, len);
 	wep->Length = len;
 	wep->KeyIndex = key_idx;
+
 	if (set_tx)
 		wep->KeyIndex |= 1 << 31;
 #if 0 /* Setting bit30 does not seem to work with some NDIS drivers */
@@ -1059,7 +1147,7 @@ static int wpa_driver_ralink_add_wep(struct wpa_driver_ralink_data *drv,
 	memcpy(wep->KeyMaterial, key, key_len);
 
 	wpa_hexdump_key(MSG_MSGDUMP, "RALINK: OID_802_11_ADD_WEP",
-			(char *) wep, len);
+			(const u8 *) wep, len);
 	res = ralink_set_oid(drv, OID_802_11_ADD_WEP, (char *) wep, len);
 
 	free(wep);
@@ -1081,22 +1169,27 @@ static int wpa_driver_ralink_set_key(void *priv, wpa_alg alg, const u8 *addr,
     wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 
 	if (addr == NULL || memcmp(addr, "\xff\xff\xff\xff\xff\xff",
-				   ETH_ALEN) == 0) {
+				   ETH_ALEN) == 0)
+	{
 		/* Group Key */
 		pairwise = 0;
 		wpa_driver_ralink_get_bssid(drv, bssid);
-	} else {
+	}
+	else
+	{
 		/* Pairwise Key */
 		pairwise = 1;
 		memcpy(bssid, addr, ETH_ALEN);
 	}
 
-	if (alg == WPA_ALG_NONE || key_len == 0) {
+	if (alg == WPA_ALG_NONE || key_len == 0)
+	{
 		return wpa_driver_ralink_remove_key(drv, key_idx, addr, bssid,
 						  pairwise);
 	}
 
-	if (alg == WPA_ALG_WEP) {
+	if (alg == WPA_ALG_WEP)
+	{
 		return wpa_driver_ralink_add_wep(drv, pairwise, key_idx, set_tx,
 					       key, key_len);
 	}
@@ -1104,34 +1197,45 @@ static int wpa_driver_ralink_set_key(void *priv, wpa_alg alg, const u8 *addr,
 	len = 12 + 6 + 6 + 8 + key_len;
 
 	nkey = malloc(len);
+
 	if (nkey == NULL)
 		return -1;
+
 	memset(nkey, 0, len);
 
 	nkey->Length = len;
 	nkey->KeyIndex = key_idx;
+
 	if (set_tx)
 		nkey->KeyIndex |= 1 << 31;
+
 	if (pairwise)
 		nkey->KeyIndex |= 1 << 30;
+
 	if (seq && seq_len)
 		nkey->KeyIndex |= 1 << 29;
+
 	nkey->KeyLength = key_len;
 	memcpy(nkey->BSSID, bssid, ETH_ALEN);
-	if (seq && seq_len) {
+
+	if (seq && seq_len)
+	{
 		for (i = 0; i < seq_len; i++)
 			nkey->KeyRSC |= seq[i] << (i * 8);
 	}
-	if (alg == WPA_ALG_TKIP && key_len == 32) {
+	if (alg == WPA_ALG_TKIP && key_len == 32)
+	{
 		memcpy(nkey->KeyMaterial, key, 16);
 		memcpy(nkey->KeyMaterial + 16, key + 24, 8);
 		memcpy(nkey->KeyMaterial + 24, key + 16, 8);
-	} else {
+	}
+	else
+	{
 		memcpy(nkey->KeyMaterial, key, key_len);
 	}
 
 	wpa_hexdump_key(MSG_MSGDUMP, "RALINK: OID_802_11_ADD_KEY",
-			(char *) nkey, len);
+			(const u8 *) nkey, len);
 	res = ralink_set_oid(drv, OID_802_11_ADD_KEY, (char *) nkey, len);
 	free(nkey);
 
@@ -1145,7 +1249,8 @@ static int wpa_driver_ralink_disassociate(void *priv, const u8 *addr,
 
 	wpa_printf(MSG_DEBUG, "%s", __FUNCTION__);
 	if (ralink_set_oid(drv, OID_802_11_DISASSOCIATE,
-			 "    ", 4) < 0) {
+			 "    ", 4) < 0)
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: Failed to set "
 			   "OID_802_11_DISASSOCIATE");
 	}
@@ -1177,28 +1282,37 @@ wpa_driver_ralink_associate(void *priv,
 		mode = Ndis802_11IBSS;
 	else
 		mode = Ndis802_11Infrastructure;
+
 	if (ralink_set_oid(drv, OID_802_11_INFRASTRUCTURE_MODE,
-			 (char *) &mode, sizeof(mode)) < 0) {
+			 (char *) &mode, sizeof(mode)) < 0)
+	{
 		wpa_printf(MSG_DEBUG, "RALINK: Failed to set "
 			   "OID_802_11_INFRASTRUCTURE_MODE (%d)",
 			   (int) mode);
 		/* Try to continue anyway */
 	}
 
-	if (params->wpa_ie == NULL || params->wpa_ie_len == 0) {
-		if (params->auth_alg & AUTH_ALG_SHARED_KEY) {
+	if (params->wpa_ie == NULL || params->wpa_ie_len == 0)
+	{
+		if (params->auth_alg & AUTH_ALG_SHARED_KEY)
+		{
 			if (params->auth_alg & AUTH_ALG_OPEN_SYSTEM)
 				auth_mode = Ndis802_11AuthModeAutoSwitch;
 			else
 				auth_mode = Ndis802_11AuthModeShared;
-		} else
+		}
+		else
 			auth_mode = Ndis802_11AuthModeOpen;
-	} else if (params->wpa_ie[0] == RSN_INFO_ELEM) {
+	}
+	else if (params->wpa_ie[0] == RSN_INFO_ELEM)
+	{
 		if (params->key_mgmt_suite == KEY_MGMT_PSK)
 			auth_mode = Ndis802_11AuthModeWPA2PSK;
 		else
 			auth_mode = Ndis802_11AuthModeWPA2;
-	} else {
+	}
+	else
+	{
 		if (params->key_mgmt_suite == KEY_MGMT_WPA_NONE)
 			auth_mode = Ndis802_11AuthModeWPANone;
 		else if (params->key_mgmt_suite == KEY_MGMT_PSK)
@@ -1207,27 +1321,28 @@ wpa_driver_ralink_associate(void *priv,
 			auth_mode = Ndis802_11AuthModeWPA;
 	}
 
-	switch (params->pairwise_suite) {
-	case CIPHER_CCMP:
-		encr = Ndis802_11Encryption3Enabled;
-		break;
-	case CIPHER_TKIP:
-		encr = Ndis802_11Encryption2Enabled;
-		break;
-	case CIPHER_WEP40:
-	case CIPHER_WEP104:
-		encr = Ndis802_11Encryption1Enabled;
-		break;
-	case CIPHER_NONE:
-		if (params->group_suite == CIPHER_CCMP)
+	switch (params->pairwise_suite)
+	{
+		case CIPHER_CCMP:
 			encr = Ndis802_11Encryption3Enabled;
-		else if (params->group_suite == CIPHER_TKIP)
-			encr = Ndis802_11Encryption2Enabled;
-        else
-            encr = Ndis802_11EncryptionDisabled;
 		break;
-	default:
-		encr = Ndis802_11EncryptionDisabled;
+		case CIPHER_TKIP:
+			encr = Ndis802_11Encryption2Enabled;
+		break;
+		case CIPHER_WEP40:
+		case CIPHER_WEP104:
+			encr = Ndis802_11Encryption1Enabled;
+		break;
+		case CIPHER_NONE:
+			if (params->group_suite == CIPHER_CCMP)
+				encr = Ndis802_11Encryption3Enabled;
+			else if (params->group_suite == CIPHER_TKIP)
+				encr = Ndis802_11Encryption2Enabled;
+        		else
+            			encr = Ndis802_11EncryptionDisabled;
+		break;
+		default:
+			encr = Ndis802_11EncryptionDisabled;
 	};
        
 	ralink_set_auth_mode(drv, auth_mode);
